@@ -1,15 +1,15 @@
-// Copyright (c) 2018-2019 Intel Corporation
-// 
+// Copyright (c) 2018-2020 Intel Corporation
+//
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
 // in the Software without restriction, including without limitation the rights
 // to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 // copies of the Software, and to permit persons to whom the Software is
 // furnished to do so, subject to the following conditions:
-// 
+//
 // The above copyright notice and this permission notice shall be included in all
 // copies or substantial portions of the Software.
-// 
+//
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
@@ -38,13 +38,6 @@
 #include <list>
 #include <assert.h>
 
-#ifndef MFX_MAX
-#define MFX_MAX( a, b ) ( ((a) > (b)) ? (a) : (b) )
-#endif
-#ifndef MFX_MIN
-#define MFX_MIN( a, b ) ( ((a) < (b)) ? (a) : (b) )
-#endif
-
 #define MFX_SORT_COMMON(_AR, _SZ, _COND)\
     for (mfxU32 _i = 0; _i < (_SZ); _i ++)\
         for (mfxU32 _j = _i; _j < (_SZ); _j ++)\
@@ -73,8 +66,6 @@ template<class T> inline void Zero(T & obj)                   { memset(&obj, 0,
 template<class T> inline void Zero(std::vector<T> & vec)      { memset(&vec[0], 0, sizeof(T) * vec.size()); }
 template<class T> inline void Zero(T * first, size_t cnt)     { memset(first, 0, sizeof(T) * cnt); }
 template<class T> inline T Abs  (T x)               { return (x > 0 ? x : -x); }
-template<class T> inline T Min  (T x, T y)          { return MFX_MIN(x, y); }
-template<class T> inline T Max  (T x, T y)          { return MFX_MAX(x, y); }
 
 template<class T> bool AlignDown(T& value, mfxU32 alignment)
 {
@@ -163,7 +154,7 @@ constexpr mfxU8    IDX_INVALID           = 0xff;
 constexpr mfxU8    HW_SURF_ALIGN_W       = 16;
 constexpr mfxU8    HW_SURF_ALIGN_H       = 16;
 
-constexpr mfxU8    MAX_SLICES            = 200; // WA for driver issue regarding CNL and older platforms
+constexpr mfxU16   MAX_SLICES            = 600; // conforms to level 6 limits
 constexpr mfxU8    DEFAULT_LTR_INTERVAL  = 16;
 constexpr mfxU8    DEFAULT_PPYR_INTERVAL = 3;
 
@@ -204,16 +195,6 @@ enum
     INSERT_SEI = (INSERT_BPSEI | INSERT_PTSEI | INSERT_DCVSEI | INSERT_LLISEI)
 };
 
-inline bool IsOn(mfxU32 opt)
-{
-    return opt == MFX_CODINGOPTION_ON;
-}
-
-inline bool IsOff(mfxU32 opt)
-{
-    return opt == MFX_CODINGOPTION_OFF;
-}
-
 inline mfxStatus GetWorstSts(mfxStatus sts1, mfxStatus sts2)
 {
     // WRN statuses > 0, ERR statuses < 0, ERR_NONE = 0
@@ -325,7 +306,7 @@ struct RoiData{
     mfxU32  Right;
     mfxU32  Bottom;
 
-    mfxI16  Priority;
+    mfxI16  DeltaQP;
 };
 
 struct Task : DpbFrame
@@ -336,7 +317,7 @@ struct Task : DpbFrame
     Slice             m_sh                            = {};
 
     mfxU32            m_idxBs                         = IDX_INVALID;
-    mfxU8             m_idxCUQp                       = IDX_INVALID;
+    mfxU32            m_idxCUQp                       = IDX_INVALID;
     bool              m_bCUQPMap                      = false;
 
     mfxU8             m_refPicList[2][MAX_DPB_SIZE]   = {};
@@ -432,6 +413,10 @@ namespace ExtBuffer
          MFX_EXTBUFF_MBQP,
          MFX_EXTBUFF_ENCODER_ROI,
          MFX_EXTBUFF_DIRTY_RECTANGLES,
+#ifdef MFX_ENABLE_HEVCE_HDR_SEI
+         MFX_EXTBUFF_MASTERING_DISPLAY_COLOUR_VOLUME,
+         MFX_EXTBUFF_CONTENT_LIGHT_LEVEL_INFO,
+#endif
          MFX_EXTBUFF_ENCODED_FRAME_INFO
     };
 
@@ -458,9 +443,13 @@ namespace ExtBuffer
         EXTBUF(mfxExtDirtyRect,             MFX_EXTBUFF_DIRTY_RECTANGLES);
         EXTBUF(mfxExtBRC,                   MFX_EXTBUFF_BRC);
         EXTBUF(mfxExtMBQP,                  MFX_EXTBUFF_MBQP);
-#if defined(MFX_ENABLE_HEVCE_WEIGHTED_PREDICTION)
+#ifdef MFX_ENABLE_HEVCE_WEIGHTED_PREDICTION
         EXTBUF(mfxExtPredWeightTable,       MFX_EXTBUFF_PRED_WEIGHT_TABLE);
-#endif //defined(MFX_ENABLE_HEVCE_WEIGHTED_PREDICTION)
+#endif
+#ifdef MFX_ENABLE_HEVCE_HDR_SEI
+        EXTBUF(mfxExtMasteringDisplayColourVolume, MFX_EXTBUFF_MASTERING_DISPLAY_COLOUR_VOLUME);
+        EXTBUF(mfxExtContentLightLevelInfo, MFX_EXTBUFF_CONTENT_LIGHT_LEVEL_INFO);
+#endif
         EXTBUF(mfxExtAVCEncodedFrameInfo, MFX_EXTBUFF_ENCODED_FRAME_INFO);
 
     #undef EXTBUF
@@ -552,6 +541,10 @@ namespace ExtBuffer
 #endif //defined(MFX_ENABLE_HEVCE_WEIGHTED_PREDICTION)
 #if (MFX_VERSION >= 1025)
         MFX_COPY_FIELD(EnableNalUnitType);
+#endif
+#if MFX_VERSION >= MFX_VERSION_NEXT
+        MFX_COPY_FIELD(DeblockingAlphaTcOffset);
+        MFX_COPY_FIELD(DeblockingBetaOffset);
 #endif
         MFX_COPY_FIELD(LowDelayBRC);
     }
@@ -730,7 +723,7 @@ public:
             }
         }
 
-        m_numTL = Max<mfxU8>(m_numTL, 1);
+        m_numTL = std::max<mfxU8>(m_numTL, 1);
     }
 
     mfxU8 NumTL() const { return m_numTL; }
@@ -792,6 +785,10 @@ public:
         mfxExtBRC                   extBRC;
         mfxExtEncoderROI            ROI;
         mfxExtDirtyRect             DirtyRect;
+#ifdef MFX_ENABLE_HEVCE_HDR_SEI
+        mfxExtMasteringDisplayColourVolume   DisplayColour;
+        mfxExtContentLightLevelInfo LightLevel;
+#endif
         mfxExtEncoderResetOption   ResetOpt;
         mfxExtBuffer *              m_extParam[SIZE_OF_ARRAY(ExtBuffer::allowed_buffers)];
     } m_ext;
