@@ -65,8 +65,7 @@ namespace MfxHwVP9Encode
         mfxStatus CreateAuxilliaryDevice(
             VideoCORE* core,
             GUID       guid,
-            mfxU32     width,
-            mfxU32     height);
+            VP9MfxVideoParam const & par);
 
         virtual
         mfxStatus CreateAccelerationService(
