@@ -305,16 +305,6 @@ mfxStatus SetSupportedParameters(mfxVideoParam & par)
     return MFX_ERR_NONE;
 }
 
-inline bool IsOn(mfxU32 opt)
-{
-    return opt == MFX_CODINGOPTION_ON;
-}
-
-inline bool IsOff(mfxU32 opt)
-{
-    return opt == MFX_CODINGOPTION_OFF;
-}
-
 struct Bool
 {
     Bool(bool initValue) : value(initValue) {};
@@ -609,31 +599,47 @@ bool CheckAndFixQIndexDelta(mfxI16& qIndexDelta, mfxU16 qIndex)
     return Clamp(qIndexDelta, minQIdxDelta, maxQIdxDelta);
 }
 
-mfxStatus CheckPerSegmentParams(mfxVP9SegmentParam& segPar, ENCODE_CAPS_VP9 const & caps, mfxU16 QP)
+mfxStatus CheckPerSegmentParams(mfxVP9SegmentParam& segPar, ENCODE_CAPS_VP9 const & caps, mfxInfoMFX &video_par, mfxEncodeCtrl *ctrl_par = nullptr)
 {
     Bool changed = false;
     mfxU16& features = segPar.FeatureEnabled;
 
-    // check QIndex feature
-    if (false == CheckFeature(features, &segPar.QIndexDelta, FEAT_QIDX, caps))
-    {
-        changed = true;
-    }
-
-    // if delta Q index value is out of valid range - just ignore it
-    if (false == CheckRangeDflt(segPar.QIndexDelta, -MAX_Q_INDEX, MAX_Q_INDEX, 0))
-    {
-        changed = true;
-    }
-    else
-    {
-        // if delta Q index value is OK, but Q index value + delta is out of valid range - clamp Q index delta
-        if (false == CheckAndFixQIndexDelta(segPar.QIndexDelta, QP))
+    if (video_par.RateControlMethod == MFX_RATECONTROL_CQP) {
+        // check QIndex feature
+        if (false == CheckFeature(features, &segPar.QIndexDelta, FEAT_QIDX, caps))
         {
             changed = true;
         }
-    }
 
+        // if delta Q index value is out of valid range - just ignore it
+        if (false == CheckRangeDflt(segPar.QIndexDelta, -MAX_Q_INDEX, MAX_Q_INDEX, 0))
+        {
+            changed = true;
+        }
+        else
+        {
+            // if delta Q index value is OK, but Q index value + delta is out of valid range - clamp Q index delta
+            if (false == CheckAndFixQIndexDelta(segPar.QIndexDelta, video_par.QPI))
+            {
+                changed = true;
+            }
+
+            // if delta Q index value is OK, but Q index value + global QP + frame QP delta is out of valid range - clamp Q index delta
+            if (ctrl_par)
+            {
+                mfxExtVP9Param* pExtParRuntime = GetExtBuffer(*ctrl_par);
+                if (pExtParRuntime)
+                {
+                    if (!CheckAndFixQIndexDelta(segPar.QIndexDelta, video_par.QPI + pExtParRuntime->QIndexDeltaLumaDC) ||
+                        !CheckAndFixQIndexDelta(segPar.QIndexDelta, video_par.QPI + pExtParRuntime->QIndexDeltaChromaAC) ||
+                        !CheckAndFixQIndexDelta(segPar.QIndexDelta, video_par.QPI + pExtParRuntime->QIndexDeltaChromaDC))
+                    {
+                        changed = true;
+                    }
+                }
+            }
+        }
+    }
 
     // check LF Level feature
     if (false == CheckFeature(features, &segPar.LoopFilterLevelDelta, FEAT_LF_LVL, caps))
@@ -708,11 +714,13 @@ inline void ConvertStatusToBools(Bool& changed, Bool& unsupported, mfxStatus sts
 }
 
 // if called with only video_par - assumed stream init, if both video_par and ctrl_par - assumed on-frame checking
-mfxStatus CheckSegmentationParam(mfxExtVP9Segmentation& seg, mfxU32 frameWidth, mfxU32 frameHeight, ENCODE_CAPS_VP9 const & caps, mfxInfoMFX &video_par, mfxEncodeCtrl *ctrl_par = nullptr)
+mfxStatus CheckSegmentationParam(mfxExtVP9Segmentation& seg, mfxU32 frameWidth, mfxU32 frameHeight, ENCODE_CAPS_VP9 const & caps, VP9MfxVideoParam const &par, mfxEncodeCtrl *ctrl_par = nullptr)
 {
     Bool changed = false;
     Bool unsupported = false;
 
+    mfxInfoMFX video_par = par.mfx;
+
     if ((seg.NumSegments != 0 || true == AnyMandatorySegMapParam(seg)) && caps.ForcedSegmentationSupport == 0)
     {
         // driver/HW don't support segmentation
@@ -753,7 +761,7 @@ mfxStatus CheckSegmentationParam(mfxExtVP9Segmentation& seg, mfxU32 frameWidth,
 
     for (mfxU16 i = 0; i < seg.NumSegments; i++)
     {
-        mfxStatus sts = CheckPerSegmentParams(seg.Segment[i], caps, video_par.QPI);
+        mfxStatus sts = CheckPerSegmentParams(seg.Segment[i], caps, video_par, ctrl_par);
         if (sts == MFX_WRN_INCOMPATIBLE_VIDEO_PARAM)
         {
             changed = true;
@@ -1015,9 +1023,14 @@ mfxStatus CheckParameters(VP9MfxVideoParam &par, ENCODE_CAPS_VP9 const &caps)
         unsupported = true;
     }
 
-    if (fi.Width > caps.MaxPicWidth || fi.Height > caps.MaxPicHeight)
+    if (fi.Width > caps.MaxPicWidth)
     {
         fi.Width = 0;
+        unsupported = true;
+    }
+
+    if (fi.Height > caps.MaxPicHeight)
+    {
         fi.Height = 0;
         unsupported = true;
     }
@@ -1367,6 +1380,20 @@ mfxStatus CheckParameters(VP9MfxVideoParam &par, ENCODE_CAPS_VP9 const &caps)
          unsupported = true;
      }
 
+     if (extPar.FrameHeight &&
+         extPar.FrameHeight > caps.MaxPicHeight)
+     {
+         extPar.FrameHeight = 0;
+         unsupported = true;
+     }
+
+     if (extPar.FrameWidth &&
+         extPar.FrameWidth > caps.MaxPicWidth)
+     {
+         extPar.FrameWidth = 0;
+         unsupported = true;
+     }
+
      if (extPar.FrameWidth && fi.Width &&
          extPar.FrameWidth > fi.Width)
      {
@@ -1457,7 +1484,7 @@ mfxStatus CheckParameters(VP9MfxVideoParam &par, ENCODE_CAPS_VP9 const &caps)
 
     mfxExtVP9Segmentation& seg = GetExtBufferRef(par);
 
-    mfxStatus segSts = CheckSegmentationParam(seg, width, height, caps, par.mfx);
+    mfxStatus segSts = CheckSegmentationParam(seg, width, height, caps, par);
     ConvertStatusToBools(changed, unsupported, segSts);
 
     if (IsOn(opt2.MBBRC) && seg.NumSegments)
@@ -1478,8 +1505,8 @@ mfxStatus CheckParameters(VP9MfxVideoParam &par, ENCODE_CAPS_VP9 const &caps)
 
     if (rows && height)
     {
-        mfxU16 heightInSBs = static_cast<mfxU16>(CeilDiv(height, SB_SIZE));
-        mfxU16 maxPossibleRows = MFX_MIN(heightInSBs, MAX_NUM_TILE_ROWS);
+        mfxU16 heightInTiles = static_cast<mfxU16>(CeilDiv(height, MIN_TILE_HEIGHT));
+        mfxU16 maxPossibleRows = std::min<mfxU16>(heightInTiles, MAX_NUM_TILE_ROWS);
         if (rows > maxPossibleRows)
         {
             rows = maxPossibleRows;
@@ -1499,7 +1526,7 @@ mfxStatus CheckParameters(VP9MfxVideoParam &par, ENCODE_CAPS_VP9 const &caps)
     if (cols && width)
     {
         mfxU16 widthInMinTileCols = static_cast<mfxU16>(CeilDiv(width, MIN_TILE_WIDTH));
-        mfxU16 maxPossibleCols = MFX_MIN(widthInMinTileCols, MAX_NUM_TILES);
+        mfxU16 maxPossibleCols = std::min<mfxU16>(widthInMinTileCols, MAX_NUM_TILES);
         if (cols > maxPossibleCols)
         {
             cols = maxPossibleCols;
@@ -1669,7 +1696,7 @@ mfxStatus SetDefaults(
     mfxExtVP9Param& extPar = GetExtBufferRef(par);
     if (extPar.FrameWidth)
     {
-        SetDefault(fi.CropW, MFX_MIN(fi.Width, extPar.FrameWidth));
+        SetDefault(fi.CropW, std::min(fi.Width, extPar.FrameWidth));
     }
     else
     {
@@ -1678,7 +1705,7 @@ mfxStatus SetDefaults(
     }
     if (extPar.FrameHeight)
     {
-        SetDefault(fi.CropH, MFX_MIN(fi.Height, extPar.FrameHeight));
+        SetDefault(fi.CropH, std::min(fi.Height, extPar.FrameHeight));
     }
     else
     {
@@ -1895,9 +1922,6 @@ mfxStatus CheckSurface(
     MFX_CHECK(surface.Info.Width <= initWidth, MFX_ERR_INVALID_VIDEO_PARAM);
     MFX_CHECK(surface.Info.Height <= initHeight, MFX_ERR_INVALID_VIDEO_PARAM);
 
-    mfxU32 pitch = (surface.Data.PitchHigh << 16) + surface.Data.PitchLow;
-    MFX_CHECK(pitch < 0x8000, MFX_ERR_UNDEFINED_BEHAVIOR);
-
     return MFX_ERR_NONE;
 }
 
@@ -1963,9 +1987,7 @@ mfxStatus CheckAndFixCtrl(
         {
             const mfxExtVP9Param& extPar = GetExtBufferRef(video);
 
-            mfxInfoMFX video_par_tmp = video.mfx;
-
-            sts = CheckSegmentationParam(*seg, extPar.FrameWidth, extPar.FrameHeight, caps, video_par_tmp, &ctrl);
+            sts = CheckSegmentationParam(*seg, extPar.FrameWidth, extPar.FrameHeight, caps, video, &ctrl);
             if (sts == MFX_ERR_UNSUPPORTED ||
                 (true == AnyMandatorySegMapParam(*seg) && false == AllMandatorySegMapParams(*seg)) ||
                 IsOn(opt2.MBBRC))
@@ -2002,6 +2024,14 @@ mfxStatus CheckAndFixCtrl(
         }
     }
 
+    mfxExtVP9Segmentation* seg_video = GetExtBuffer(video);
+    if (seg_video)
+    {
+        const mfxExtVP9Param& extParSeg = GetExtBufferRef(video);
+        mfxStatus sts = CheckSegmentationParam(*seg_video, extParSeg.FrameWidth, extParSeg.FrameHeight, caps, video, &ctrl);
+        MFX_CHECK_STS(sts);
+    }
+
     return checkSts;
 }
 
