@@ -1,4 +1,4 @@
-// Copyright (c) 2018-2019 Intel Corporation
+// Copyright (c) 2018-2020 Intel Corporation
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -52,18 +52,26 @@ static mfxU16 GetGopSize(mfxVideoParam *par)
 {
     return  par->mfx.GopPicSize == 0 ?  256 : par->mfx.GopPicSize ;
 }
-static mfxU16 GetRefDist(mfxVideoParam *par)
+static mfxU16 GetRefDist(mfxVideoParam *par, eMFXHWType targetHw)
 {
-    mfxExtLAControl *pControl = (mfxExtLAControl *) GetExtBuffer(par->ExtParam, par->NumExtParam, MFX_EXTBUFF_LOOKAHEAD_CTRL);
     mfxU16 GopSize = GetGopSize(par);
-    mfxU16 refDist = par->mfx.GopRefDist == 0 ? ((pControl && pControl->BPyramid == MFX_CODINGOPTION_ON) ? 8 : 3) : par->mfx.GopRefDist;
-    return refDist <= GopSize ? refDist: GopSize;
+    mfxU16 refDist = par->mfx.GopRefDist;
+    if (refDist == 0)
+    {
+        mfxExtLAControl *pControl = (mfxExtLAControl *)GetExtBuffer(par->ExtParam, par->NumExtParam, MFX_EXTBUFF_LOOKAHEAD_CTRL);
+        refDist = (
+		    IsOn(par->mfx.LowPower)
+         && targetHw <= MFX_HW_TGL_LP )
+		  ? 1 
+		  : ((pControl && IsOn(pControl->BPyramid)) ? 8 : 3);
+    }
+    return (std::min)(refDist, GopSize);
 }
 static mfxU16 GetAsyncDeph(mfxVideoParam *par)
 {
     return par->AsyncDepth == 0 ? 3 : par->AsyncDepth;
 }
-static mfxStatus InitEncoderParameters(mfxVideoParam *par_in, mfxVideoParam *par_enc)
+static mfxStatus InitEncoderParameters(mfxVideoParam *par_in, mfxVideoParam *par_enc, eMFXHWType targetHw)
 {
     mfxExtLAControl *pControl = (mfxExtLAControl *) GetExtBuffer(par_in->ExtParam, par_in->NumExtParam, MFX_EXTBUFF_LOOKAHEAD_CTRL);
 
@@ -77,7 +85,7 @@ static mfxStatus InitEncoderParameters(mfxVideoParam *par_in, mfxVideoParam *par
     par_enc->AsyncDepth = GetAsyncDeph(par_in);
     par_enc->IOPattern  = par_in->IOPattern;
     par_enc->mfx.GopPicSize = GetGopSize(par_in);
-    par_enc->mfx.GopRefDist = GetRefDist(par_in);
+    par_enc->mfx.GopRefDist = GetRefDist(par_in, targetHw);
     par_enc->mfx.RateControlMethod = MFX_RATECONTROL_LA;
     par_enc->mfx.NumRefFrame = 2;
     par_enc->mfx.FrameInfo = par_in->mfx.FrameInfo;  
@@ -274,9 +282,9 @@ mfxStatus VideoENC_LA::Query(VideoCORE* pCore, mfxVideoParam *in, mfxVideoParam
     return MFX_ERR_NONE;
 }
 
-mfxStatus VideoENC_LA::QueryIOSurf(VideoCORE* , mfxVideoParam *par, mfxFrameAllocRequest *request)
+mfxStatus VideoENC_LA::QueryIOSurf(VideoCORE* core, mfxVideoParam *par, mfxFrameAllocRequest *request)
 {
-    MFX_CHECK_NULL_PTR2(par,request);
+    MFX_CHECK_NULL_PTR3(par,request,core);
 
     mfxExtLAControl *pControl = (mfxExtLAControl *) GetExtBuffer(par->ExtParam, par->NumExtParam, MFX_EXTBUFF_LOOKAHEAD_CTRL);
 
@@ -304,7 +312,7 @@ mfxStatus VideoENC_LA::QueryIOSurf(VideoCORE* , mfxVideoParam *par, mfxFrameAllo
             ? MFX_MEMTYPE_OPAQUE_FRAME
             : MFX_MEMTYPE_EXTERNAL_FRAME;
     }
-    request->NumFrameMin         = GetRefDist(par) + GetAsyncDeph(par) + pControl->LookAheadDepth;
+    request->NumFrameMin         = GetRefDist(par, core->GetHWType()) + GetAsyncDeph(par) + pControl->LookAheadDepth;
     request->NumFrameSuggested   = request->NumFrameMin;
     request->Info                = par->mfx.FrameInfo;
 
@@ -447,7 +455,7 @@ mfxStatus VideoENC_LA::Init(mfxVideoParam *par)
         ddi_opt.LaScaleFactor = m_LaControl.DownScaleFactor = CheckScaleFactor(m_LaControl.DownScaleFactor, par->mfx.TargetUsage, par->mfx.FrameInfo.Width > 4000);
         ddi_opt.LookAheadDependency =  m_LaControl.DependencyDepth = CheckLookAheadDependency(m_LaControl.DependencyDepth,par->mfx.TargetUsage);
 
-        MFX_CHECK_STS (InitEncoderParameters(par, &par_enc));    
+        MFX_CHECK_STS (InitEncoderParameters(par, &par_enc, m_core->GetHWType()));
         m_video = par_enc;
     
          MFX_ENCODE_CAPS hwCaps = {};
@@ -752,7 +760,8 @@ mfxFrameSurface1 * VideoENC_LA::GetFrameToVME()
         return 0;
 }
 
-
+//SubmitFrameLARoutine and QueryFrameLARoutine may be needed if we use a double entry point
+#if 0
 static mfxStatus SubmitFrameLARoutine(void *pState, void * pParam, mfxU32 /*threadNumber*/, mfxU32 /*callNumber*/)
 {
     mfxStatus tskRes;
@@ -782,6 +791,8 @@ static mfxStatus QueryFrameLARoutine(void *pState, void *pParam, mfxU32 /*thread
     return tskRes;
 
 } // mfxStatus RunFrameVPPRoutine(void *pState, void *pParam, mfxU32 threadNumber, mfxU32 callNumber)
+#endif
+
 static mfxStatus RunFrameVPPRoutine(void *pState, void *pParam, mfxU32 /*threadNumber*/, mfxU32 /*callNumber*/)
 {
     mfxStatus sts = MFX_ERR_NONE;
@@ -964,19 +975,10 @@ mfxStatus VideoENC_LA::RunFrameVmeENCCheck(
         numEntryPoints = 1;
     }
     return sts;
-} 
-
-mfxF64 const QSTEP[52] = {
-        0.630,  0.707,  0.794,  0.891,  1.000,   1.122,   1.260,   1.414,   1.587,   1.782,   2.000,   2.245,   2.520,
-        2.828,  3.175,  3.564,  4.000,  4.490,   5.040,   5.657,   6.350,   7.127,   8.000,   8.980,  10.079,  11.314,
-    12.699, 14.254, 16.000, 17.959, 20.159,  22.627,  25.398,  28.509,  32.000,  35.919,  40.317,  45.255,  50.797,
-    57.018, 64.000, 71.838, 80.635, 90.510, 101.594, 114.035, 128.000, 143.675, 161.270, 181.019, 203.187, 228.070
-};
+}
 
 namespace MfxHwH264EncodeHW
 {
-    //MfxHwH264Encode::VmeData * FindUnusedVmeData(std::vector<MfxHwH264Encode::VmeData> & vmeData);
-    mfxF64 const INTRA_QSTEP_COEFF  = 2.0;
     mfxU8 GetSkippedQp(MfxHwH264Encode::MbData const & mb);
     void DivideCost(std::vector<MfxHwH264Encode::MbData> & mb, mfxI32 width, mfxI32 height, mfxU32 cost, mfxI32 x, mfxI32 y);
 };
@@ -1645,11 +1647,10 @@ namespace MfxHwH264EncodeHW
         mfxU32 pocL0,
         mfxU32 pocL1)
     {
-        mfxI32 tb = MFX_MIN(MFX_MAX(-128, mfxI32(pocCur - pocL0)), 127);
-        mfxI32 td = MFX_MIN(MFX_MAX(-128, mfxI32(pocL1  - pocL0)), 127);
+        mfxI32 tb = mfx::clamp(mfxI32(pocCur - pocL0), -128, 127);
+        mfxI32 td = mfx::clamp(mfxI32(pocL1  - pocL0), -128, 127);
         mfxI32 tx = (16384 + abs(td/2)) / td;
-        mfxI32 distScaleFactor = MFX_MIN(MFX_MAX(-1024, (tb * tx + 32) >> 6), 1023);
-        return distScaleFactor;
+        return mfx::clamp((tb * tx + 32) >> 6, -1024, 1023);
     }
 
 
@@ -1948,6 +1949,8 @@ mfxStatus CmContextLA::QueryVme(sLADdiTask const & task,
 {
     MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_INTERNAL, "QueryVme");
 
+    MFX_CHECK_NULL_PTR1(e);
+
     INT status = e->WaitForTaskFinished();
     if (status == CM_EXCEED_MAX_TIMEOUT)
         return MFX_ERR_GPU_HANG;
@@ -1957,6 +1960,8 @@ mfxStatus CmContextLA::QueryVme(sLADdiTask const & task,
     MfxHwH264Encode::LAOutObject * cmMb = (MfxHwH264Encode::LAOutObject *)task.m_cmMbSys;
     MfxHwH264Encode::VmeData *      cur  = task.m_Curr.VmeData;
 
+    MFX_CHECK_NULL_PTR2(cmMb, cur);
+
     { MFX_AUTO_LTRACE(MFX_TRACE_LEVEL_INTERNAL, "Compensate costs");
     MfxHwH264Encode::mfxVMEUNIIn const & costs = SelectCosts(task.m_TaskInfo.InputFrame.frameType);
     for (size_t i = 0; i < cur->mb.size(); i++)
@@ -1979,7 +1984,7 @@ mfxStatus CmContextLA::QueryVme(sLADdiTask const & task,
             mfxU32 modeCostLambda = Map44LutValueBack(costs.ModeCost[LUTMODE_INTER_16x16]);
             mfxU32 mvCostLambda   = (task.m_TaskInfo.InputFrame.frameType & MFX_FRAMETYPE_P)
                 ? GetVmeMvCostP(m_lutMvP, mb) : GetVmeMvCostB(m_lutMvB, mb);
-            mfxU16 bitCostLambda = mfxU16(MFX_MIN(mb.interCost, modeCostLambda + mvCostLambda));
+            mfxU16 bitCostLambda = mfxU16(std::min<mfxU32>(mb.interCost, modeCostLambda + mvCostLambda));
             mb.dist = mb.interCost - bitCostLambda;
         }
     }
@@ -1995,7 +2000,7 @@ mfxStatus CmContextLA::QueryVme(sLADdiTask const & task,
     for (size_t i = 0; i < cur->mb.size(); i++)
     {
         cur->mb[i].intraCost     = cmMb[i].intraCost;
-        cur->mb[i].interCost     = MFX_MIN(cmMb[i].intraCost, cmMb[i].interCost);
+        cur->mb[i].interCost     = std::min(cmMb[i].intraCost, cmMb[i].interCost);
         cur->mb[i].intraMbFlag   = cmMb[i].IntraMbFlag;
         cur->mb[i].skipMbFlag    = cmMb[i].SkipMbFlag;
         cur->mb[i].mbType        = cmMb[i].MbType5Bits;
