@@ -1,4 +1,4 @@
-// Copyright (c) 2018 Intel Corporation
+// Copyright (c) 2020 Intel Corporation
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -127,7 +127,6 @@ typedef struct {
 #define MIN(a, b) (((a) < (b)) ? (a) : (b))
 #define ABS(a)    (((a) < 0) ? (-(a)) : (a))
 
-#define CLIPVAL(VAL, MINVAL, MAXVAL) MAX(MINVAL, MIN(MAXVAL, VAL))
 #define MCTF_CHECK_CM_ERR(STS, ERR) if ((STS) != CM_SUCCESS) { ASC_PRINTF("FAILED at file: %s, line: %d, cmerr: %d\n", __FILE__, __LINE__, STS); return ERR; }
 
 #define CHECK_ERR(ERR) if ((ERR) != PASSED) { ASC_PRINTF("FAILED at file: %s, line: %d\n", __FILE__, __LINE__); return (ERR); }
@@ -360,6 +359,8 @@ private:
     t_MCTF_LOAD     pMCTF_LOAD_func;
     t_MCTF_SPDEN    pMCTF_SpDen_func;
 
+    eMFXHWType
+        mctf_HWType;
     CmDevice
         * device;
     CmQueue
@@ -624,6 +625,11 @@ private:
     );
     mfxU8  SetOverlapOp();
     mfxU8  SetOverlapOp_half();
+    mfxI32 MCTF_Enqueue(
+        CmTask* taskInt,
+        CmEvent* & eInt,
+        const CmThreadSpace* tSInt = 0
+    );    
     mfxI32 MCTF_RUN_TASK_NA(
         CmKernel * kernel,
         bool       reset,
