@@ -1,4 +1,4 @@
-// Copyright (c) 2019 Intel Corporation
+// Copyright (c) 2020 Intel Corporation
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -37,6 +37,9 @@
 #include "genx_me_gen11lp_isa.h"
 #include "genx_mc_gen11lp_isa.h"
 #include "genx_sd_gen11lp_isa.h"
+#include "genx_me_gen12lp_isa.h"
+#include "genx_mc_gen12lp_isa.h"
+#include "genx_sd_gen12lp_isa.h"
 
 #include <algorithm>
 #include <climits>
@@ -625,11 +628,12 @@ mfxStatus CMC::MCTF_SET_ENV(
     if (!device)
         return MFX_ERR_NOT_INITIALIZED;
 
+    mctf_HWType = core->GetHWType();
     hwSize = 4;
     res = device->GetCaps(CAP_GPU_PLATFORM, hwSize, &hwType);
     MCTF_CHECK_CM_ERR(res, MFX_ERR_DEVICE_FAILED);
 
-    if(core->GetHWType() >= MFX_HW_ICL)
+    if(mctf_HWType >= MFX_HW_ICL)
         res = device->CreateQueueEx(queue, CM_VME_QUEUE_CREATE_OPTION);
     else
         res = device->CreateQueue(queue);
@@ -747,6 +751,8 @@ mfxStatus CMC::MCTF_SET_ENV(
         res = device->LoadProgram((void *)genx_me_gen11, sizeof(genx_me_gen11), programMe, "nojitter");
     else if (hwType == PLATFORM_INTEL_ICLLP)
         res = device->LoadProgram((void *)genx_me_gen11lp, sizeof(genx_me_gen11lp), programMe, "nojitter");
+    else if (hwType == PLATFORM_INTEL_TGLLP)
+        res = device->LoadProgram((void *)genx_me_gen12lp, sizeof(genx_me_gen12lp), programMe, "nojitter");
     else if (hwType >= PLATFORM_INTEL_SKL && hwType <= PLATFORM_INTEL_CFL)
         res = device->LoadProgram((void *)genx_me_gen9, sizeof(genx_me_gen9), programMe, "nojitter");
     else
@@ -778,6 +784,8 @@ mfxStatus CMC::MCTF_SET_ENV(
         res = device->LoadProgram((void *)genx_mc_gen11, sizeof(genx_mc_gen11), programMc, "nojitter");
     else if (hwType == PLATFORM_INTEL_ICLLP)
         res = device->LoadProgram((void *)genx_mc_gen11lp, sizeof(genx_mc_gen11lp), programMc, "nojitter");
+    else if (hwType == PLATFORM_INTEL_TGLLP)
+        res = device->LoadProgram((void *)genx_mc_gen12lp, sizeof(genx_mc_gen12lp), programMc, "nojitter");
     else if (hwType >= PLATFORM_INTEL_SKL && hwType <= PLATFORM_INTEL_CFL)
         res = device->LoadProgram((void *)genx_mc_gen9, sizeof(genx_mc_gen9), programMc, "nojitter");
     else
@@ -792,6 +800,8 @@ mfxStatus CMC::MCTF_SET_ENV(
         res = device->LoadProgram((void *)genx_sd_gen11, sizeof(genx_sd_gen11), programDe, "nojitter");
     else if (hwType == PLATFORM_INTEL_ICLLP)
         res = device->LoadProgram((void *)genx_sd_gen11lp, sizeof(genx_sd_gen11lp), programDe, "nojitter");
+    else if (hwType == PLATFORM_INTEL_TGLLP)
+        res = device->LoadProgram((void *)genx_sd_gen12lp, sizeof(genx_sd_gen12lp), programDe, "nojitter");
     else if (hwType >= PLATFORM_INTEL_SKL && hwType <= PLATFORM_INTEL_CFL)
         res = device->LoadProgram((void *)genx_sd_gen9, sizeof(genx_sd_gen9), programDe, "nojitter");
     else
@@ -1393,6 +1403,24 @@ mfxI32 CMC::MCTF_SET_KERNELDe(
     return res;
 }
 
+mfxI32 CMC::MCTF_Enqueue(
+    CmTask* taskInt,
+    CmEvent* & eInt,
+    const CmThreadSpace* tSInt
+)
+{
+    mfxI32
+        ret = CM_FAILURE;
+        switch (mctf_HWType){
+        case MFX_HW_ICL:
+            ret = queue->EnqueueFast(taskInt, eInt, tSInt);
+            break;
+        default:
+            ret = queue->Enqueue(taskInt, eInt, tSInt);
+        }
+    return ret;
+}
+
 mfxI32 CMC::MCTF_RUN_TASK_NA(
     CmKernel * kernel,
     bool       reset,
@@ -1420,7 +1448,7 @@ mfxI32 CMC::MCTF_RUN_TASK_NA(
     }
     res = task->AddKernel(kernel);
     MCTF_CHECK_CM_ERR(res, res);
-    res = queue->Enqueue(task, e);
+    res = MCTF_Enqueue(task, e);
     MCTF_CHECK_CM_ERR(res, res);
     return res;
 }
@@ -1489,7 +1517,7 @@ mfxI32 CMC::MCTF_RUN_DOUBLE_TASK(
     MCTF_CHECK_CM_ERR(res, res);
     res = task->AddKernel(mcKernel);
     MCTF_CHECK_CM_ERR(res, res);
-    res = queue->Enqueue(task, e);
+    res = MCTF_Enqueue(task, e);
     MCTF_CHECK_CM_ERR(res, res);
     return res;
 }
@@ -1518,7 +1546,7 @@ mfxI32 CMC::MCTF_RUN_MCTASK(
     }
     res = task->AddKernel(kernel);
     MCTF_CHECK_CM_ERR(res, res);
-    res = queue->Enqueue(task, e, threadSpaceMC2);
+    res = MCTF_Enqueue(task, e, threadSpaceMC2);
     MCTF_CHECK_CM_ERR(res, res);
     return res;
 }
@@ -1547,7 +1575,7 @@ mfxI32 CMC::MCTF_RUN_TASK(
     }
     res = task->AddKernel(kernel);
     MCTF_CHECK_CM_ERR(res, res);
-    res = queue->Enqueue(task, e, tS);
+    res = MCTF_Enqueue(task, e, tS);
     MCTF_CHECK_CM_ERR(res, res);
     res = device->DestroyThreadSpace(tS);
     return res;
@@ -1702,7 +1730,7 @@ mfxI32 CMC::MCTF_RUN_ME_MC_H(
 
     if (pMCTF_NOA_func)
     {
-        res = queue->Enqueue(task, e);
+        res = MCTF_Enqueue(task, e);
         MCTF_CHECK_CM_ERR(res, res);
         res = e->WaitForTaskFinished();
         MCTF_CHECK_CM_ERR(res, res);
@@ -2532,7 +2560,7 @@ mfxI32 CMC::MCTF_RUN_Denoise(mfxU16 srcNum)
     threadSpace = 0;
     res = MCTF_RUN_TASK(kernelMcDen, task != 0);
     MCTF_CHECK_CM_ERR(res, res);
-    res = queue->Enqueue(task, e);
+    res = MCTF_Enqueue(task, e);
     MCTF_CHECK_CM_ERR(res, res);
 
     if (tsWidthFull > CM_MAX_THREADSPACE_WIDTH_FOR_MW)
@@ -2550,7 +2578,7 @@ mfxI32 CMC::MCTF_RUN_Denoise(mfxU16 srcNum)
         // the rest of frame TS
         res = MCTF_RUN_TASK(kernelMcDen, task != 0);
         MCTF_CHECK_CM_ERR(res, res);
-        res = queue->Enqueue(task, e);
+        res = MCTF_Enqueue(task, e);
         MCTF_CHECK_CM_ERR(res, res);
     }
     res = e->WaitForTaskFinished();
