@@ -258,23 +258,6 @@ namespace MfxHwH264Encode
     template <class T> struct GetPointedType<T *> { typedef T Type; };
     template <class T> struct GetPointedType<T const *> { typedef T Type; };
 
-
-    inline bool IsOn(mfxU32 opt)
-    {
-        return opt == MFX_CODINGOPTION_ON;
-    }
-
-    inline bool IsOff(mfxU32 opt)
-    {
-        return opt == MFX_CODINGOPTION_OFF;
-    }
-
-    inline bool IsAdapt(mfxU32 opt)
-    {
-        return opt == MFX_CODINGOPTION_ADAPTIVE;
-    }
-
-
     struct HrdParameters
     {
         mfxU8   cpbCntMinus1;
@@ -622,6 +605,7 @@ namespace MfxHwH264Encode
 
             mfxU16 widthLa;
             mfxU16 heightLa;
+            mfxU32 TCBRCTargetFrameSize;
         } calcParam;
     };
 
@@ -780,6 +764,7 @@ namespace MfxHwH264Encode
     void InheritDefaultValues(
         MfxVideoParam const & parInit,
         MfxVideoParam &       parReset,
+        MFX_ENCODE_CAPS const & hwCaps,
         mfxVideoParam const * parResetIn = 0);
 
     mfxStatus CheckPayloads(
@@ -917,6 +902,21 @@ namespace MfxHwH264Encode
         }
     }
 
+    inline mfxU32 GetAvgFrameSizeInBytes(const MfxVideoParam &  par, bool bMaxKbps)
+    {
+        return mfxU32(1000.0 / 8.0*((par.mfx.MaxKbps && bMaxKbps) ? par.mfx.MaxKbps : par.mfx.TargetKbps) * std::max<mfxU32>(par.mfx.BRCParamMultiplier,1) /
+            (mfxF64(par.mfx.FrameInfo.FrameRateExtN) / par.mfx.FrameInfo.FrameRateExtD) );
+    }
+    inline bool IsTCBRC(const MfxVideoParam &  par, MFX_ENCODE_CAPS const & hwCaps)
+    {
+        mfxExtCodingOption3 &extOpt3 = GetExtBufferRef(par);
+        mfxExtCodingOption  &extOpt = GetExtBufferRef(par);
+        return (IsOn(extOpt3.LowDelayBRC) && (hwCaps.ddi_caps.TCBRCSupport) && IsOff(extOpt.NalHrdConformance) &&
+               (par.mfx.RateControlMethod  ==  MFX_RATECONTROL_VBR  ||
+                par.mfx.RateControlMethod  ==  MFX_RATECONTROL_QVBR ||
+                par.mfx.RateControlMethod  ==  MFX_RATECONTROL_VCM ));
+    }
+
     inline mfxU8 GetPayloadLayout(mfxU32 fieldPicFlag, mfxU32 secondFieldPicFlag)
     {
         return fieldPicFlag == 0
@@ -931,11 +931,16 @@ namespace MfxHwH264Encode
         return GetPayloadLayout(fp.AVC.FieldPicFlag, fp.AVC.SecondFieldPicFlag);
     }
 
+    inline bool IsDriverSliceSizeControlEnabled(const MfxVideoParam & par, MFX_ENCODE_CAPS const & hwCaps)
+    {
+        return IsOn(par.mfx.LowPower) && hwCaps.ddi_caps.SliceLevelRateCtrl;
+    }
+
     mfxU8 ConvertFrameTypeMfx2Ddi(mfxU32 type);
 
     mfxU8 ConvertMfxFrameType2SliceType(mfxU8 type);
 
-    ENCODE_FRAME_SIZE_TOLERANCE ConvertLowDelayBRCMfx2Ddi(mfxU16 type);
+    ENCODE_FRAME_SIZE_TOLERANCE ConvertLowDelayBRCMfx2Ddi(mfxU16 type, bool bTCBRC);
 
     struct MfxMemId
     {
