@@ -1,4 +1,4 @@
-// Copyright (c) 2018 Intel Corporation
+// Copyright (c) 2018-2019 Intel Corporation
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -453,13 +453,10 @@ mfxStatus MFXVideoDECODE_Close(mfxSession session)
 
     try
     {
-        if (!session->m_pDECODE)
-        {
-            return MFX_ERR_NOT_INITIALIZED;
-        }
+        MFX_CHECK(session->m_pDECODE, MFX_ERR_NOT_INITIALIZED);
 
         // wait until all tasks are processed
-        session->m_pScheduler->WaitForTaskCompletion(session->m_pDECODE.get());
+        session->m_pScheduler->WaitForAllTasksCompletion(session->m_pDECODE.get());
 
         mfxRes = session->m_pDECODE->Close();
         // delete the codec's instance if not plugin
@@ -476,6 +473,8 @@ mfxStatus MFXVideoDECODE_Close(mfxSession session)
     }
 
     MFX_LTRACE_I(MFX_TRACE_LEVEL_API, mfxRes);
+
+    MFX_CHECK_STS(mfxRes);
     return mfxRes;
 }
 
@@ -502,10 +501,7 @@ mfxStatus MFXVideoDECODE_DecodeFrameAsync(mfxSession session, mfxBitstream *bs,
 
         // Wait for the bit stream
         mfxRes = session->m_pScheduler->WaitForDependencyResolved(bs);
-        if (MFX_ERR_NONE != mfxRes)
-        {
-            return mfxRes;
-        }
+        MFX_CHECK_STS(mfxRes);
 
         // reset the sync point
         *syncp = NULL;
@@ -513,6 +509,8 @@ mfxStatus MFXVideoDECODE_DecodeFrameAsync(mfxSession session, mfxBitstream *bs,
 
         memset(&task, 0, sizeof(MFX_TASK));
         mfxRes = session->m_pDECODE->DecodeFrameCheck(bs, surface_work, surface_out, &task.entryPoint);
+        MFX_CHECK(mfxRes >= 0 || MFX_ERR_MORE_DATA_SUBMIT_TASK == static_cast<int>(mfxRes), mfxRes);
+
         // source data is OK, go forward
         if (task.entryPoint.pRoutine)
         {
@@ -533,7 +531,7 @@ mfxStatus MFXVideoDECODE_DecodeFrameAsync(mfxSession session, mfxBitstream *bs,
                 {
                     session->m_plgDec.get()->GetPlugin(plugin);
                     MFX_CHECK_STS(plugin.GetPluginParam(plugin.pthis, &par));
-                    if (!memcmp(&MFX_PLUGINID_HEVCD_SW, &par.PluginUID, sizeof(MFX_PLUGINID_HEVCD_SW)))
+                    if (MFX_PLUGINID_HEVCD_SW == par.PluginUID)
                     {
                         task.pDst[0] = 0;
                     }
@@ -547,10 +545,7 @@ mfxStatus MFXVideoDECODE_DecodeFrameAsync(mfxSession session, mfxBitstream *bs,
 
             // register input and call the task
             mfxAddRes = session->m_pScheduler->AddTask(task, &syncPoint);
-            if (MFX_ERR_NONE != mfxAddRes)
-            {
-                return mfxAddRes;
-            }
+            MFX_CHECK_STS(mfxAddRes);
         }
 
         if (MFX_ERR_MORE_DATA_SUBMIT_TASK == static_cast<int>(mfxRes))
