@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2018 Intel Corporation
+// Copyright (c) 2017-2020 Intel Corporation
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -46,7 +46,7 @@ typedef struct tagENCODE_QUERY_STATUS_DATA_tmp
 
 using namespace MfxHwMpeg2Encode;
 
-mfxStatus MfxHwMpeg2Encode::QueryHwCaps(VideoCORE* pCore, ENCODE_CAPS & hwCaps)
+mfxStatus MfxHwMpeg2Encode::QueryHwCaps(VideoCORE* pCore, ENCODE_CAPS & hwCaps, mfxU16 codecProfile)
 {
     EncodeHWCaps* pEncodeCaps = QueryCoreInterface<EncodeHWCaps>(pCore);
     if (!pEncodeCaps)
@@ -62,8 +62,10 @@ mfxStatus MfxHwMpeg2Encode::QueryHwCaps(VideoCORE* pCore, ENCODE_CAPS & hwCaps)
     ddi.reset( CreatePlatformMpeg2Encoder(pCore) );
     if(ddi.get() == NULL)
         return MFX_ERR_NULL_PTR;
-    mfxStatus sts = ddi.get()->QueryEncodeCaps(hwCaps);
+
+    mfxStatus sts = ddi->CreateAuxilliaryDevice(codecProfile);
     MFX_CHECK_STS(sts);
+    ddi->QueryEncodeCaps(hwCaps);
 
     return pEncodeCaps->SetHWCaps<ENCODE_CAPS>(DXVA2_Intel_Encode_MPEG2, &hwCaps);
 }
@@ -186,7 +188,7 @@ mfxStatus ExecuteBuffers::Init(const mfxVideoParamEx_MPEG2* par, mfxU32 funcId,
         m_sps.FrameRateExtD = (USHORT) fr_codeD;
         m_sps.FrameRateExtN = (USHORT) fr_codeN;
 
-        mfxU32 multiplier = MFX_MAX(par->mfxVideoParams.mfx.BRCParamMultiplier, 1);
+        mfxU32 multiplier = std::max<mfxU32>(par->mfxVideoParams.mfx.BRCParamMultiplier, 1);
 
         m_sps.bit_rate = (par->mfxVideoParams.mfx.RateControlMethod != MFX_RATECONTROL_CQP) ?
                                                             par->mfxVideoParams.mfx.TargetKbps * multiplier : 0;
