@@ -43,6 +43,7 @@
  #include "genx_fcopy_gen10_isa.h"
  #include "genx_fcopy_gen11_isa.h"
  #include "genx_fcopy_gen11lp_isa.h"
+ #include "genx_fcopy_gen12lp_isa.h"
 
 #ifdef MFX_ENABLE_MCTF
 #include "mctf_common.h"
@@ -405,14 +406,12 @@ namespace MfxHwVideoProcessing
         ReleaseResource* CreateSubResourceForMode30i60p(void);
 
         //-------------------------------------------------
-        mfxU32   GetNumToRemove( void )
+        mfxU32 GetNumToRemove( void )
         {
-            mfxU32 numFramesToRemove = m_inputFramesOrFieldPerCycle - MFX_MIN(m_inputFramesOrFieldPerCycle, m_bkwdRefCountRequired - m_bkwdRefCount);
-
             if (m_fieldWeaving)
-                numFramesToRemove = 2;
+                return 2;
 
-            return numFramesToRemove;
+            return m_inputFramesOrFieldPerCycle - std::min(m_inputFramesOrFieldPerCycle, m_bkwdRefCountRequired - m_bkwdRefCount);
         }
 
         mfxU32   GetNextBkwdRefCount( void )
@@ -423,9 +422,8 @@ namespace MfxHwVideoProcessing
             }
 
             mfxU32 numBkwdRef = m_bkwdRefCount + (m_inputFramesOrFieldPerCycle - GetNumToRemove());
-            numBkwdRef = MFX_MIN(numBkwdRef,  m_bkwdRefCountRequired);
 
-            return numBkwdRef;
+            return std::min(numBkwdRef,  m_bkwdRefCountRequired);
         }
 
         //-------------------------------------------------
@@ -636,8 +634,8 @@ namespace MfxHwVideoProcessing
                 m_frcRational[VPP_IN]  = frcRational[VPP_IN];
                 m_frcRational[VPP_OUT] = frcRational[VPP_OUT];
 
-                m_minDeltaTime = MFX_MIN(uint64_t(m_frcRational[VPP_IN].FrameRateExtD  * MFX_TIME_STAMP_FREQUENCY) / (2 * m_frcRational[VPP_IN].FrameRateExtN),
-                                         uint64_t(m_frcRational[VPP_OUT].FrameRateExtD * MFX_TIME_STAMP_FREQUENCY) / (2 * m_frcRational[VPP_OUT].FrameRateExtN));
+                m_minDeltaTime = std::min(uint64_t(m_frcRational[VPP_IN].FrameRateExtD  * MFX_TIME_STAMP_FREQUENCY) / (2 * m_frcRational[VPP_IN].FrameRateExtN),
+                                          uint64_t(m_frcRational[VPP_OUT].FrameRateExtD * MFX_TIME_STAMP_FREQUENCY) / (2 * m_frcRational[VPP_OUT].FrameRateExtN));
             }
 
             mfxStatus DoCpuFRC_AndUpdatePTS(
