@@ -1,4 +1,4 @@
-// Copyright (c) 2017 Intel Corporation
+// Copyright (c) 2017-2019 Intel Corporation
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -196,13 +196,6 @@ void ConvertCaps2ListDoUse(MfxHwVideoProcessing::mfxVppCaps& caps, std::vector<m
 
 //mfxStatus QueryExtParams()
 
-// ToDo: we have 5 copies of IsOn() function. 
-// It should be moved into api/include/mfxstructures.h
-inline bool IsOn(mfxU16 opt)
-{
-    return opt == MFX_CODINGOPTION_ON;
-}
-
 #endif // __MFX_VPP_UTILS_H
 
 #endif // MFX_ENABLE_VPP
