@@ -1,4 +1,4 @@
-// Copyright (c) 2018-2019 Intel Corporation
+// Copyright (c) 2018-2020 Intel Corporation
 // 
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -275,7 +275,6 @@ namespace MfxHwVideoProcessing
                ,Contrast(0)
                ,Hue(0)
                ,Saturation(0)
-               ,bSceneDetectionEnable(false)
                ,bVarianceEnable(false)
                ,bImgStabilizationEnable(false)
                ,istabMode(0)
@@ -298,6 +297,7 @@ namespace MfxHwVideoProcessing
                ,bEOS(false)
                ,mirroring(0)
                ,mirroringPosition(0)
+               ,mirroringExt(false)
                ,scene(VPP_NO_SCENE_CHANGE)
                ,bDeinterlace30i60p(false)
 #ifdef MFX_ENABLE_MCTF
@@ -341,7 +341,6 @@ namespace MfxHwVideoProcessing
                     detailFactor != 0 ||
                     iTargetInterlacingMode != 0 ||
                     bEnableProcAmp != false ||
-                    bSceneDetectionEnable != false ||
                     bVarianceEnable != false ||
                     bImgStabilizationEnable != false ||
                     bFRCEnable != false ||
@@ -351,6 +350,7 @@ namespace MfxHwVideoProcessing
                     rotation != 0 ||
                     scalingMode != MFX_SCALING_MODE_DEFAULT ||
                     mirroring != 0 ||
+                    mirroringExt != false ||
                     scene != VPP_NO_SCENE_CHANGE ||
                     bDeinterlace30i60p != false
 #if (MFX_VERSION >= 1025)
@@ -396,8 +396,6 @@ namespace MfxHwVideoProcessing
         mfxF64         Hue;
         mfxF64         Saturation;
 
-        bool           bSceneDetectionEnable;
-
         bool           bVarianceEnable;
         bool           bImgStabilizationEnable;
         mfxU32         istabMode;
@@ -434,6 +432,7 @@ namespace MfxHwVideoProcessing
 
         int        mirroring;
         int        mirroringPosition;
+        bool       mirroringExt;
 
         vppScene    scene;     // Keep information about scene change
         bool        bDeinterlace30i60p;
