@@ -674,8 +674,9 @@ private:
 inline int32_t CalculateDPBSize(uint8_t & level_idc, int32_t width, int32_t height, uint32_t num_ref_frames)
 {
     uint32_t dpbSize;
+    auto constexpr INTERNAL_MAX_LEVEL = H264VideoDecoderParams::H264_LEVEL_MAX + 1;
 
-    num_ref_frames = MFX_MIN(16, num_ref_frames);
+    num_ref_frames = std::min(16u, num_ref_frames);
 
     for (;;)
     {
@@ -722,7 +723,13 @@ inline int32_t CalculateDPBSize(uint8_t & level_idc, int32_t width, int32_t heig
             MaxDPBMbs = 184320;
             break;
         default:
-            MaxDPBMbs = 184320; //get as much as we may
+            // We don't support level greater than 5.2 but
+            // relax resolution constrains up to 4K, hence
+            // use value 696320 which is from level 6+ for
+            // the calculation of the DPB size when level_idc
+            // reaches 5.1+ but dpbSize is still less
+            // than num_ref_frames
+            MaxDPBMbs = 696320;
             break;
         }
 
@@ -732,9 +739,9 @@ inline int32_t CalculateDPBSize(uint8_t & level_idc, int32_t width, int32_t heig
         }
 
         uint32_t dpbLevel = MaxDPBMbs*256 / (width * height);
-        dpbSize = MFX_MIN(16, dpbLevel);
+        dpbSize = std::min(16u, dpbLevel);
 
-        if (num_ref_frames <= dpbSize || level_idc == H264VideoDecoderParams::H264_LEVEL_MAX)
+        if (num_ref_frames <= dpbSize || level_idc == INTERNAL_MAX_LEVEL)
             break;
 
         switch (level_idc) // increase level_idc
@@ -771,15 +778,22 @@ inline int32_t CalculateDPBSize(uint8_t & level_idc, int32_t width, int32_t heig
         case H264VideoDecoderParams::H264_LEVEL_5:
             level_idc = H264VideoDecoderParams::H264_LEVEL_51;
             break;
+
+        // Set level_idc to INTERNAL_MAX_LEVEL so that MaxDPBMbs = 696320
+        // can be used to calculate the DPB size.
         case H264VideoDecoderParams::H264_LEVEL_51:
         case H264VideoDecoderParams::H264_LEVEL_52:
-            level_idc = H264VideoDecoderParams::H264_LEVEL_MAX;
+            level_idc = INTERNAL_MAX_LEVEL;
             break;
         default:
             throw h264_exception(UMC_ERR_FAILED);
         }
     }
 
+    // Restore level_idc to H264_LEVEL_MAX
+    if (level_idc == INTERNAL_MAX_LEVEL)
+      level_idc = H264VideoDecoderParams::H264_LEVEL_MAX;
+
     return dpbSize;
 }
 
