@@ -1,4 +1,4 @@
-// Copyright (c) 2017-2019 Intel Corporation
+// Copyright (c) 2017-2020 Intel Corporation
 // 
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -396,7 +396,7 @@ void H265HeadersBitstream::xDecodeScalingList(H265ScalingList *scalingList, unsi
 {
     VM_ASSERT(scalingList);
 
-    int32_t i,coefNum = MFX_MIN(MAX_MATRIX_COEF_NUM,(int32_t)g_scalingListSize[sizeId]);
+    int32_t i,coefNum = std::min(MAX_MATRIX_COEF_NUM,(int32_t)g_scalingListSize[sizeId]);
     int32_t nextCoef = SCALING_LIST_START_VALUE;
     const uint16_t *scan  = (sizeId == 0) ? ScanTableDiag4x4 : g_sigLastScanCG32x32;
     int32_t *dst = scalingList->getScalingListAddress(sizeId, listId);
@@ -728,7 +728,7 @@ UMC::Status H265HeadersBitstream::GetSequenceParamSet(H265SeqParamSet *pcSPS)
     pcSPS->m_maxTrSize = 1 << pcSPS->log2_max_transform_block_size;
 
     uint32_t CtbLog2SizeY = pcSPS->log2_max_luma_coding_block_size;
-    if (pcSPS->log2_max_transform_block_size > MFX_MIN(5, CtbLog2SizeY))
+    if (pcSPS->log2_max_transform_block_size > std::min(5u, CtbLog2SizeY))
         throw h265_exception(UMC::UMC_ERR_INVALID_STREAM);
 
     pcSPS->max_transform_hierarchy_depth_inter = GetVLCElementU();
@@ -775,12 +775,12 @@ UMC::Status H265HeadersBitstream::GetSequenceParamSet(H265SeqParamSet *pcSPS)
 
         pcSPS->log2_min_pcm_luma_coding_block_size = GetVLCElementU() + 3;
 
-        if (pcSPS->log2_min_pcm_luma_coding_block_size < MFX_MIN(MinCbLog2SizeY, 5) || pcSPS->log2_min_pcm_luma_coding_block_size > MFX_MIN(CtbLog2SizeY, 5))
+        if (pcSPS->log2_min_pcm_luma_coding_block_size < std::min(MinCbLog2SizeY, 5u) || pcSPS->log2_min_pcm_luma_coding_block_size > std::min(CtbLog2SizeY, 5u))
             throw h265_exception(UMC::UMC_ERR_INVALID_STREAM);
 
         pcSPS->log2_max_pcm_luma_coding_block_size = GetVLCElementU() + pcSPS->log2_min_pcm_luma_coding_block_size;
 
-        if (pcSPS->log2_max_pcm_luma_coding_block_size > MFX_MIN(CtbLog2SizeY, 5))
+        if (pcSPS->log2_max_pcm_luma_coding_block_size > std::min(CtbLog2SizeY, 5u))
             throw h265_exception(UMC::UMC_ERR_INVALID_STREAM);
 
         pcSPS->pcm_loop_filter_disabled_flag = Get1Bit();
@@ -897,14 +897,14 @@ UMC::Status H265HeadersBitstream::GetSequenceParamSet(H265SeqParamSet *pcSPS)
                                 pcSPS->m_paletteInitializers[i * pcSPS->sps_num_palette_predictor_initializer + j] = GetBits(num_bits);
                             }
                     }
-
-                    pcSPS->motion_vector_resolution_control_idc = GetBits(2);
-                    if (pcSPS->motion_vector_resolution_control_idc == 3)
-                        //value of 3 for motion_vector_resolution_control_idc is reserved for future use by spec.
-                        throw h265_exception(UMC::UMC_ERR_INVALID_STREAM);
-
-                    pcSPS->intra_boundary_filtering_disabled_flag = Get1Bit();
                 }
+
+                pcSPS->motion_vector_resolution_control_idc = GetBits(2);
+                if (pcSPS->motion_vector_resolution_control_idc == 3)
+                    //value of 3 for motion_vector_resolution_control_idc is reserved for future use by spec.
+                    throw h265_exception(UMC::UMC_ERR_INVALID_STREAM);
+
+                pcSPS->intra_boundary_filtering_disabled_flag = Get1Bit();
             }
         }
 
@@ -1965,6 +1965,11 @@ void H265HeadersBitstream::decodeSlice(H265Slice *pSlice, const H265SeqParamSet
 
         uint32_t PicHeightInCtbsY = sps->HeightInCU;
 
+        if (sliceHdr->num_entry_point_offsets > 440)
+        {
+            vm_string_printf(VM_STRING("slice has more than 440 offsets:%d\n"), sliceHdr->num_entry_point_offsets);
+        }
+
         if (!pps->tiles_enabled_flag && pps->entropy_coding_sync_enabled_flag && sliceHdr->num_entry_point_offsets > PicHeightInCtbsY)
             throw h265_exception(UMC::UMC_ERR_INVALID_STREAM);
 
