@@ -1,4 +1,4 @@
-// Copyright (c) 2018-2019 Intel Corporation
+// Copyright (c) 2018-2020 Intel Corporation
 //
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
@@ -156,7 +156,8 @@ UMC::Status DecReferencePictureMarking_H265::UpdateRefPicMarking(ViewItem_H265 &
                     }
                     else
                     {
-                        if ((pTmp->m_PicOrderCnt % (1 << pSlice->GetSeqParam()->log2_max_pic_order_cnt_lsb)) == rps->getPOC(i) % (1 << pSlice->GetSeqParam()->log2_max_pic_order_cnt_lsb))
+                        int const MaxPicOrderCntLsb = 1 << pSlice->GetSeqParam()->log2_max_pic_order_cnt_lsb;
+                        if ((pTmp->m_PicOrderCnt & (MaxPicOrderCntLsb - 1)) == (rps->getPOC(i) & (MaxPicOrderCntLsb - 1)))
                         {
                             isReferenced = true;
                             pTmp->SetisLongTermRef(true);
@@ -1454,7 +1455,7 @@ void TaskSupplier_H265::PostProcessDisplayFrame(H265DecoderFrame *pFrame)
 
     DEBUG_PRINT((VM_STRING("Outputted %s, pppp - %d\n"), GetFrameInfoString(pFrame), pppp++));
 
-    m_maxUIDWhenWasDisplayed = MFX_MAX(m_maxUIDWhenWasDisplayed, pFrame->m_maxUIDWhenWasDisplayed);
+    m_maxUIDWhenWasDisplayed = std::max(m_maxUIDWhenWasDisplayed, pFrame->m_maxUIDWhenWasDisplayed);
 }
 
 // Find a next frame ready to be output from decoder
@@ -1689,7 +1690,7 @@ UMC::Status TaskSupplier_H265::AddOneFrame(UMC::MediaData * pSource)
     }
 
     if (m_checkCRAInsideResetProcess && !pSource)
-        return UMC::UMC_ERR_FAILED;
+        return UMC::UMC_ERR_NOT_ENOUGH_DATA;
 
     size_t moveToSpsOffset = m_checkCRAInsideResetProcess ? pSource->GetDataSize() : 0;
 
@@ -1926,6 +1927,33 @@ H265Slice *TaskSupplier_H265::DecodeSliceHeader(UMC::MediaDataEx *nalUnit)
     m_Headers.m_SeqParams.SetCurrentID(pps->pps_seq_parameter_set_id);
     m_Headers.m_PicParams.SetCurrentID(pps->pps_pic_parameter_set_id);
 
+    // calculate max entrypoints number
+    H265SeqParamSet const* sps = pSlice->GetSeqParam();
+    if (pps->entropy_coding_sync_enabled_flag)
+    {
+        int NumOfMaxEntryPoints = 0;
+        int PicHeightInCtbsY = sps->HeightInCU;
+
+        if (pps->tiles_enabled_flag == 0)
+            NumOfMaxEntryPoints = PicHeightInCtbsY;
+        else
+            NumOfMaxEntryPoints = PicHeightInCtbsY * (pps->num_tile_columns);
+
+        //reallocate memory for slice header
+        if (NumOfMaxEntryPoints > 512)
+        {
+            vm_string_printf(VM_STRING("Re-allocate buffer for slice header!\n"));
+
+            //offset_len_minus1[0-31], assume maximum 31[4 bytes]
+            int newsize = 1024 + NumOfMaxEntryPoints * 4 + DEFAULT_NU_TAIL_SIZE;
+            pSlice->m_source.Allocate(newsize);
+
+            //swap buffer again, since buffer is small at first time
+            memCopy.SetData(nalUnit);
+            swapper->SwapMemory(&pSlice->m_source, &memCopy, &removed_offsets);
+        }
+    }
+
     pSlice->m_pCurrentFrame = NULL;
 
     memory_leak_preventing.ClearNotification();
@@ -1970,7 +1998,7 @@ H265Slice *TaskSupplier_H265::DecodeSliceHeader(UMC::MediaDataEx *nalUnit)
     uint32_t currOffset = sliceHdr->m_HeaderBitstreamOffset;
     uint32_t currOffsetWithEmul = currOffset;
 
-    size_t headersEmuls = 0;
+    uint32_t headersEmuls = 0;
     for (; headersEmuls < removed_offsets.size(); headersEmuls++)
     {
         if (removed_offsets[headersEmuls] < currOffsetWithEmul)
@@ -1979,6 +2007,8 @@ H265Slice *TaskSupplier_H265::DecodeSliceHeader(UMC::MediaDataEx *nalUnit)
             break;
     }
 
+    pSlice->m_NumEmuPrevnBytesInSliceHdr = headersEmuls;
+
     // Update entry points
     size_t offsets = removed_offsets.size();
     if ((pSlice->GetPicParam()->tiles_enabled_flag || pSlice->GetPicParam()->entropy_coding_sync_enabled_flag) &&
@@ -2009,8 +2039,6 @@ H265Slice *TaskSupplier_H265::DecodeSliceHeader(UMC::MediaDataEx *nalUnit)
                 offsets -= removed_bytes;
                 removed_bytes = 0;
             }
-            else
-                pSlice->m_tileByteLocation[tile] = pSlice->m_tileByteLocation[tile] - removed_bytes;
         }
     }
 
@@ -2109,7 +2137,7 @@ void TaskSupplier_H265::CheckCRAOrBLA(const H265Slice *pSlice)
         {
             if (pCurr->isDisplayable() && !pCurr->wasOutputted())
             {
-                minRefPicResetCount = MFX_MIN(minRefPicResetCount, pCurr->RefPicListResetCount());
+                minRefPicResetCount = std::min(minRefPicResetCount, pCurr->RefPicListResetCount());
             }
         }
 
@@ -2362,7 +2390,7 @@ UMC::Status TaskSupplier_H265::InitFreeFrame(H265DecoderFrame * pFrame, const H2
     bit_depth_luma = (uint8_t) pSeqParam->bit_depth_luma;
     bit_depth_chroma = (uint8_t) pSeqParam->bit_depth_chroma;
 
-    int32_t bit_depth = MFX_MAX(bit_depth_luma, bit_depth_chroma);
+    int32_t bit_depth = std::max(bit_depth_luma, bit_depth_chroma);
 
 //    int32_t iMBWidth = pSeqParam->frame_width_in_mbs;
     //int32_t iCUWidth = pSeqParam->WidthInCU;
@@ -2431,7 +2459,7 @@ H265DecoderFrame * TaskSupplier_H265::AllocateNewFrame(const H265Slice *pSlice)
                                     pSlice->GetSeqParam()->sps_max_dec_pic_buffering[pSlice->GetSliceHeader()->nuh_temporal_id] :
                                     view.dpbSize;
 
-    view.sps_max_num_reorder_pics = MFX_MIN(pSlice->GetSeqParam()->sps_max_num_reorder_pics[HighestTid], view.sps_max_dec_pic_buffering);
+    view.sps_max_num_reorder_pics = std::min(pSlice->GetSeqParam()->sps_max_num_reorder_pics[HighestTid], view.sps_max_dec_pic_buffering);
 
     DPBUpdate(pSlice);
 
@@ -2627,16 +2655,20 @@ int32_t CalculateDPBSize(uint32_t /*profile_idc*/, uint32_t &level_idc, int32_t
 
         uint32_t MaxLumaPs = lumaPsArray[index];
         uint32_t const maxDpbPicBuf =
+#ifndef MFX_VA
+            profile_idc != H265_PROFILE_SCC ? 6 : 7;
+#else
             6;//HW handles second version of current reference (twoVersionsOfCurrDecPicFlag) itself
+#endif
 
         uint32_t PicSizeInSamplesY = width * height;
 
         if (PicSizeInSamplesY  <=  (MaxLumaPs  >>  2 ))
-            MaxDpbSize = MFX_MIN(4 * maxDpbPicBuf, 16);
+            MaxDpbSize = std::min(4 * maxDpbPicBuf, 16u);
         else if (PicSizeInSamplesY  <=  (MaxLumaPs  >>  1 ))
-            MaxDpbSize = MFX_MIN(2 * maxDpbPicBuf, 16);
+            MaxDpbSize = std::min(2 * maxDpbPicBuf, 16u);
         else if (PicSizeInSamplesY  <=  ((3 * MaxLumaPs)  >>  2 ))
-            MaxDpbSize = MFX_MIN((4 * maxDpbPicBuf) / 3, 16);
+            MaxDpbSize = std::min((4 * maxDpbPicBuf) / 3, 16u);
         else
             MaxDpbSize = maxDpbPicBuf;
 
