@@ -1,15 +1,15 @@
-// Copyright (c) 2018 Intel Corporation
-// 
+// Copyright (c) 2018-2020 Intel Corporation
+//
 // Permission is hereby granted, free of charge, to any person obtaining a copy
 // of this software and associated documentation files (the "Software"), to deal
 // in the Software without restriction, including without limitation the rights
 // to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 // copies of the Software, and to permit persons to whom the Software is
 // furnished to do so, subject to the following conditions:
-// 
+//
 // The above copyright notice and this permission notice shall be included in all
 // copies or substantial portions of the Software.
-// 
+//
 // THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 // IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 // FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
@@ -31,6 +31,7 @@ extern "C" {
 
 
 /* Frame ID for SVC and MVC */
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxU16      TemporalId;
     mfxU16      PriorityId;
@@ -44,8 +45,10 @@ typedef struct {
         };
     };
 } mfxFrameId;
+MFX_PACK_END()
 
-#pragma pack(push, 4)
+/* This struct has 4-byte alignment for binary compatibility with previously released versions of API */
+MFX_PACK_BEGIN_USUAL_STRUCT()
 /* Frame Info */
 typedef struct {
     mfxU32  reserved[4];
@@ -84,7 +87,7 @@ typedef struct {
     mfxU16  ChromaFormat;
     mfxU16  reserved2;
 } mfxFrameInfo;
-#pragma pack(pop)
+MFX_PACK_END()
 
 /* FourCC */
 enum {
@@ -101,7 +104,7 @@ enum {
     MFX_FOURCC_P8           = 41,                                /*  D3DFMT_P8   */
     MFX_FOURCC_P8_TEXTURE   = MFX_MAKEFOURCC('P','8','M','B'),
     MFX_FOURCC_P010         = MFX_MAKEFOURCC('P','0','1','0'),
-#if (MFX_VERSION >= MFX_VERSION_NEXT)
+#if (MFX_VERSION >= 1031)
     MFX_FOURCC_P016         = MFX_MAKEFOURCC('P','0','1','6'),
 #endif
     MFX_FOURCC_P210         = MFX_MAKEFOURCC('P','2','1','0'),
@@ -117,7 +120,7 @@ enum {
     MFX_FOURCC_Y210         = MFX_MAKEFOURCC('Y','2','1','0'),
     MFX_FOURCC_Y410         = MFX_MAKEFOURCC('Y','4','1','0'),
 #endif
-#if (MFX_VERSION >= MFX_VERSION_NEXT)
+#if (MFX_VERSION >= 1031)
     MFX_FOURCC_Y216         = MFX_MAKEFOURCC('Y','2','1','6'),
     MFX_FOURCC_Y416         = MFX_MAKEFOURCC('Y','4','1','6'),
 #endif
@@ -178,7 +181,7 @@ enum {
 };
 
 #if (MFX_VERSION >= 1027)
-#pragma pack(push, 4)
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct
 {
     mfxU32 U : 10;
@@ -186,11 +189,11 @@ typedef struct
     mfxU32 V : 10;
     mfxU32 A :  2;
 } mfxY410;
-#pragma pack(pop)
+MFX_PACK_END()
 #endif
 
 #if (MFX_VERSION >= 1025)
-#pragma pack(push, 4)
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct
 {
     mfxU32 B : 10;
@@ -198,11 +201,11 @@ typedef struct
     mfxU32 R : 10;
     mfxU32 A :  2;
 } mfxA2RGB10;
-#pragma pack(pop)
-
+MFX_PACK_END()
 #endif
 
 /* Frame Data Info */
+MFX_PACK_BEGIN_STRUCT_W_L_TYPE()
 typedef struct {
     union {
         mfxExtBuffer **ExtParam;
@@ -238,7 +241,7 @@ typedef struct {
         mfxU16  *U16;
         mfxU8   *G;
 #if (MFX_VERSION >= 1027)
-        mfxY410 *Y410;          /* for.U format (merged AVYU) */
+        mfxY410 *Y410;          /* for Y410 format (merged AVYU) */
 #endif
     };
     union {
@@ -257,13 +260,16 @@ typedef struct {
     mfxU16  Corrupted;
     mfxU16  DataFlag;
 } mfxFrameData;
+MFX_PACK_END()
 
 /* Frame Surface */
+MFX_PACK_BEGIN_STRUCT_W_L_TYPE()
 typedef struct {
     mfxU32  reserved[4];
     mfxFrameInfo    Info;
     mfxFrameData    Data;
 } mfxFrameSurface1;
+MFX_PACK_END()
 
 enum {
     MFX_TIMESTAMPCALC_UNKNOWN = 0,
@@ -271,6 +277,7 @@ enum {
 };
 
 /* Transcoding Info */
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxU32  reserved[7];
 
@@ -340,13 +347,17 @@ typedef struct {
         };
     };
 } mfxInfoMFX;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxU32  reserved[8];
     mfxFrameInfo    In;
     mfxFrameInfo    Out;
 } mfxInfoVPP;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_STRUCT_W_PTR()
 typedef struct {
     mfxU32  AllocId;
     mfxU32  reserved[2];
@@ -363,6 +374,7 @@ typedef struct {
     mfxU16  NumExtParam;
     mfxU16  reserved2;
 } mfxVideoParam;
+MFX_PACK_END()
 
 /* IOPattern */
 enum {
@@ -458,6 +470,9 @@ enum {
     MFX_PROFILE_HEVC_MAIN10           =2,
     MFX_PROFILE_HEVC_MAINSP           =3,
     MFX_PROFILE_HEVC_REXT             =4,
+#if (MFX_VERSION >= 1032)
+    MFX_PROFILE_HEVC_SCC              =9,
+#endif
 
     MFX_LEVEL_HEVC_1   = 10,
     MFX_LEVEL_HEVC_2   = 20,
@@ -534,6 +549,7 @@ enum {
     MFX_TRELLIS_B       =0x08
 };
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer Header;
 
@@ -568,6 +584,7 @@ typedef struct {
     mfxU16      PicTimingSEI;           /* tri-state option */
     mfxU16      VuiNalHrdParameters;    /* tri-state option */
 } mfxExtCodingOption;
+MFX_PACK_END()
 
 enum {
     MFX_B_REF_UNKNOWN = 0,
@@ -602,6 +619,7 @@ enum {
         MFX_REFRESH_SLICE          = 3
 };
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer Header;
 
@@ -637,6 +655,7 @@ typedef struct {
     mfxU16      EnableMAD;              /* tri-state option */
     mfxU16      UseRawRef;              /* tri-state option */
 } mfxExtCodingOption2;
+MFX_PACK_END()
 
 /* WeightedPred */
 enum {
@@ -653,7 +672,10 @@ enum {
     MFX_SCENARIO_VIDEO_CONFERENCE    = 2,
     MFX_SCENARIO_ARCHIVE             = 3,
     MFX_SCENARIO_LIVE_STREAMING      = 4,
-    MFX_SCENARIO_CAMERA_CAPTURE      = 5
+    MFX_SCENARIO_CAMERA_CAPTURE      = 5,
+    MFX_SCENARIO_VIDEO_SURVEILLANCE  = 6,
+    MFX_SCENARIO_GAME_STREAMING      = 7,
+    MFX_SCENARIO_REMOTE_GAMING       = 8
 };
 
 /* ContentInfo */
@@ -695,6 +717,7 @@ enum {
 
 #endif
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer Header;
 
@@ -798,6 +821,7 @@ typedef struct {
 #endif
     mfxU16      reserved[163];
 } mfxExtCodingOption3;
+MFX_PACK_END()
 
 /* IntraPredBlockSize/InterPredBlockSize */
 enum {
@@ -906,25 +930,35 @@ enum {
     MFX_EXTBUFF_MPEG2_QUANT_MATRIX              = MFX_MAKEFOURCC('M','2','Q','M'),
     MFX_EXTBUFF_TASK_DEPENDENCY                 = MFX_MAKEFOURCC('S','Y','N','C'),
 #endif
+#if (MFX_VERSION >= 1031)
+    MFX_EXTBUFF_PARTIAL_BITSTREAM_PARAM         = MFX_MAKEFOURCC('P','B','O','P'),
+#endif
 };
 
 /* VPP Conf: Do not use certain algorithms  */
+MFX_PACK_BEGIN_STRUCT_W_PTR()
 typedef struct {
     mfxExtBuffer    Header;
     mfxU32          NumAlg;
     mfxU32*         AlgList;
 } mfxExtVPPDoNotUse;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer    Header;
     mfxU16  DenoiseFactor;
 } mfxExtVPPDenoise;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer    Header;
     mfxU16  DetailFactor;
 } mfxExtVPPDetail;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_STRUCT_W_L_TYPE()
 typedef struct {
     mfxExtBuffer    Header;
     mfxF64   Brightness;
@@ -932,15 +966,19 @@ typedef struct {
     mfxF64   Hue;
     mfxF64   Saturation;
 } mfxExtVPPProcAmp;
+MFX_PACK_END()
 
 /* statistics collected for decode, encode and vpp */
+MFX_PACK_BEGIN_STRUCT_W_L_TYPE()
 typedef struct {
     mfxU32  reserved[16];
     mfxU32  NumFrame;
     mfxU64  NumBit;
     mfxU32  NumCachedFrame;
 } mfxEncodeStat;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxU32  reserved[16];
     mfxU32  NumFrame;
@@ -948,13 +986,17 @@ typedef struct {
     mfxU32  NumError;
     mfxU32  NumCachedFrame;
 } mfxDecodeStat;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxU32  reserved[16];
     mfxU32  NumFrame;
     mfxU32  NumCachedFrame;
 } mfxVPPStat;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer    Header;
 
@@ -971,12 +1013,14 @@ typedef struct {
     mfxU16          SceneChangeRate;
     mfxU16          RepeatedFrame;
 } mfxExtVppAuxData;
+MFX_PACK_END()
 
 /* CtrlFlags */
 enum {
     MFX_PAYLOAD_CTRL_SUFFIX = 0x00000001 /* HEVC suffix SEI */
 };
 
+MFX_PACK_BEGIN_STRUCT_W_PTR()
 typedef struct {
     mfxU32      CtrlFlags;
     mfxU32      reserved[3];
@@ -985,7 +1029,9 @@ typedef struct {
     mfxU16      Type;       /* SEI message type in H.264 or user data start_code in MPEG-2 */
     mfxU16      BufSize;    /* payload buffer size in bytes */
 } mfxPayload;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_STRUCT_W_PTR()
 typedef struct {
     mfxExtBuffer    Header;
 #if (MFX_VERSION >= 1025)
@@ -1007,6 +1053,7 @@ typedef struct {
     mfxExtBuffer    **ExtParam;
     mfxPayload      **Payload;      /* for field pair, first field uses even payloads and second field uses odd payloads */
 } mfxEncodeCtrl;
+MFX_PACK_END()
 
 /* Buffer Memory Types */
 enum {
@@ -1044,6 +1091,7 @@ enum {
 #endif
 };
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     union {
         mfxU32  AllocId;
@@ -1056,7 +1104,9 @@ typedef struct {
     mfxU16  NumFrameSuggested;
     mfxU16  reserved2;
 } mfxFrameAllocRequest;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_STRUCT_W_PTR()
 typedef struct {
     mfxU32      AllocId;
     mfxU32      reserved[3];
@@ -1068,6 +1118,7 @@ typedef struct {
     mfxU16      reserved2;
 #endif
 } mfxFrameAllocResponse;
+MFX_PACK_END()
 
 /* FrameType */
 enum {
@@ -1112,9 +1163,11 @@ typedef enum {
     MFX_HANDLE_D3D11_DEVICE                     = 3,
     MFX_HANDLE_VA_DISPLAY                       = 4,
     MFX_HANDLE_RESERVED3                        = 5,
-#if (MFX_VERSION >= MFX_VERSION_NEXT)
+#if (MFX_VERSION >= 1030)
     MFX_HANDLE_VA_CONFIG_ID                     = 6,
     MFX_HANDLE_VA_CONTEXT_ID                    = 7,
+#endif
+#if (MFX_VERSION >= MFX_VERSION_NEXT)
     MFX_HANDLE_CM_DEVICE                        = 8
 #endif
 } mfxHandleType;
@@ -1125,6 +1178,7 @@ typedef enum {
     MFX_SKIPMODE_LESS=2
 } mfxSkipMode;
 
+MFX_PACK_BEGIN_STRUCT_W_PTR()
 typedef struct {
     mfxExtBuffer    Header;
     mfxU8           *SPSBuffer;
@@ -1134,7 +1188,9 @@ typedef struct {
     mfxU16          SPSId;
     mfxU16          PPSId;
 } mfxExtCodingOptionSPSPPS;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_STRUCT_W_L_TYPE()
 typedef struct {
     mfxExtBuffer    Header;
 
@@ -1147,7 +1203,9 @@ typedef struct {
 
     mfxU16          reserved[6];
 } mfxExtCodingOptionVPS;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer    Header;
     mfxU16          VideoFormat;
@@ -1157,13 +1215,17 @@ typedef struct {
     mfxU16          TransferCharacteristics;
     mfxU16          MatrixCoefficients;
 } mfxExtVideoSignalInfo;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_STRUCT_W_PTR()
 typedef struct {
     mfxExtBuffer    Header;
     mfxU32          NumAlg;
     mfxU32          *AlgList;
 } mfxExtVPPDoUse;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_STRUCT_W_PTR()
 typedef struct {
     mfxExtBuffer    Header;
     mfxU32      reserved1[2];
@@ -1174,7 +1236,9 @@ typedef struct {
         mfxU16  NumSurface;
     } In, Out;
 } mfxExtOpaqueSurfaceAlloc;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer    Header;
     mfxU16          NumRefIdxL0Active;
@@ -1191,6 +1255,7 @@ typedef struct {
     mfxU16      ApplyLongTermIdx;
     mfxU16      reserved[15];
 } mfxExtAVCRefListCtrl;
+MFX_PACK_END()
 
 enum {
     MFX_FRCALGM_PRESERVE_TIMESTAMP    = 0x0001,
@@ -1198,23 +1263,27 @@ enum {
     MFX_FRCALGM_FRAME_INTERPOLATION   = 0x0004
 };
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer    Header;
     mfxU16      Algorithm;
     mfxU16      reserved;
     mfxU32      reserved2[15];
 } mfxExtVPPFrameRateConversion;
+MFX_PACK_END()
 
 enum {
     MFX_IMAGESTAB_MODE_UPSCALE = 0x0001,
     MFX_IMAGESTAB_MODE_BOXING  = 0x0002
 };
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer    Header;
     mfxU16  Mode;
     mfxU16  reserved[11];
 } mfxExtVPPImageStab;
+MFX_PACK_END()
 
 #if (MFX_VERSION >= 1025)
 
@@ -1223,6 +1292,7 @@ enum {
     MFX_PAYLOAD_IDR = 1
 };
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer    Header;
     mfxU16      reserved[15];
@@ -1235,8 +1305,9 @@ typedef struct {
     mfxU32 MaxDisplayMasteringLuminance;
     mfxU32 MinDisplayMasteringLuminance;
 } mfxExtMasteringDisplayColourVolume;
+MFX_PACK_END()
 
-
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer    Header;
     mfxU16      reserved[9];
@@ -1245,9 +1316,10 @@ typedef struct {
     mfxU16 MaxContentLightLevel;
     mfxU16 MaxPicAverageLightLevel;
 } mfxExtContentLightLevelInfo;
-
+MFX_PACK_END()
 #endif
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
   mfxExtBuffer    Header;
   mfxU32      reserved[14];
@@ -1270,7 +1342,9 @@ typedef struct {
       mfxU32    TimeOffset;
   } TimeStamp[3];
 } mfxExtPictureTimingSEI;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer    Header;
     mfxU32          reserved1[4];
@@ -1282,41 +1356,32 @@ typedef struct {
         mfxU16 reserved[3];
     }Layer[8];
 } mfxExtAvcTemporalLayers;
+MFX_PACK_END()
 
-#if (MFX_VERSION >= MFX_VERSION_NEXT)
-
-enum {
-    MFX_MEMORY_TILING_LINEAR = 0x0001,
-    MFX_MEMORY_TILING_Y      = 0x0002,
-    MFX_MEMORY_TILING_X      = 0x0004
-};
-
-#endif
-
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer Header;
 
     mfxU32      MBPerSec;
-#if (MFX_VERSION >= MFX_VERSION_NEXT)
-    mfxU16      InputMemoryTiling;
-    mfxU16      reserved[57];
-#else
     mfxU16      reserved[58];
-#endif
 } mfxExtEncoderCapability;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer Header;
 
     mfxU16      StartNewSequence;
     mfxU16      reserved[11];
 } mfxExtEncoderResetOption;
+MFX_PACK_END()
 
 /*LongTermIdx*/
 enum {
     MFX_LONGTERM_IDX_NO_IDX = 0xFFFF
 };
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer    Header;
 
@@ -1336,7 +1401,9 @@ typedef struct {
         mfxU16      reserved[4];
     } UsedRefListL0[32], UsedRefListL1[32];
 } mfxExtAVCEncodedFrameInfo;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct mfxVPPCompInputStream {
     mfxU32  DstX;
     mfxU32  DstY;
@@ -1355,7 +1422,9 @@ typedef struct mfxVPPCompInputStream {
 
     mfxU16  reserved2[17];
 } mfxVPPCompInputStream;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_STRUCT_W_PTR()
 typedef struct {
     mfxExtBuffer    Header;
 
@@ -1378,6 +1447,7 @@ typedef struct {
     mfxU16       NumInputStream;
     mfxVPPCompInputStream *InputStream;
 } mfxExtVPPComposite;
+MFX_PACK_END()
 
 /* TransferMatrix */
 enum {
@@ -1393,6 +1463,7 @@ enum {
     MFX_NOMINALRANGE_16_235    = 2
 };
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer    Header;
     mfxU16          reserved1[4];
@@ -1412,6 +1483,7 @@ typedef struct {
         };
     };
 } mfxExtVPPVideoSignalInfo;
+MFX_PACK_END()
 
 /* ROI encoding mode */
 enum {
@@ -1419,6 +1491,7 @@ enum {
     MFX_ROI_MODE_QP_DELTA =  1
 };
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer    Header;
 
@@ -1438,6 +1511,7 @@ typedef struct {
         mfxU16  reserved2[7];
     } ROI[256];
 } mfxExtEncoderROI;
+MFX_PACK_END()
 
 /*Deinterlacing Mode*/
 enum {
@@ -1465,6 +1539,7 @@ enum {
     MFX_TELECINE_POSITION_PROVIDED    = 4
 };
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer    Header;
     mfxU16  Mode;
@@ -1472,7 +1547,9 @@ typedef struct {
     mfxU16  TelecineLocation;
     mfxU16  reserved[9];
 } mfxExtVPPDeinterlacing;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer    Header;
     mfxU16          NumRefIdxL0Active;
@@ -1486,6 +1563,7 @@ typedef struct {
     } RefPicList0[32], RefPicList1[32];
 
 }mfxExtAVCRefLists;
+MFX_PACK_END()
 
 enum {
     MFX_VPP_COPY_FRAME      =0x01,
@@ -1501,6 +1579,7 @@ enum {
     MFX_PICTYPE_BOTTOMFIELD =0x04
 };
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer    Header;
 
@@ -1509,7 +1588,9 @@ typedef struct {
     mfxU16          OutField;
     mfxU16          reserved[25];
 } mfxExtVPPFieldProcessing;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer    Header;
 
@@ -1538,7 +1619,9 @@ typedef struct {
 
     mfxU16  reserved[13];
 } mfxExtDecVideoProcessing;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer Header;
 
@@ -1547,6 +1630,7 @@ typedef struct {
     mfxU16       ChromaSampleLocTypeBottomField;
     mfxU16       reserved[9];
 } mfxExtChromaLocInfo;
+MFX_PACK_END()
 
 /* MBQPMode */
 enum {
@@ -1554,6 +1638,7 @@ enum {
     MFX_MBQP_MODE_QP_DELTA = 1
 };
 
+MFX_PACK_BEGIN_STRUCT_W_L_TYPE()
 typedef struct {
     mfxExtBuffer    Header;
 
@@ -1567,7 +1652,9 @@ typedef struct {
         mfxU64 reserved2;
     };
 } mfxExtMBQP;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_STRUCT_W_L_TYPE()
 typedef struct {
     mfxExtBuffer Header;
 
@@ -1578,7 +1665,9 @@ typedef struct {
         mfxU64  reserved2;
     };
 } mfxExtMBForceIntra;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer Header;
 
@@ -1586,7 +1675,9 @@ typedef struct {
     mfxU16 NumTileColumns;
     mfxU16 reserved[74];
 }mfxExtHEVCTiles;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_STRUCT_W_L_TYPE()
 typedef struct {
     mfxExtBuffer Header;
 
@@ -1597,9 +1688,10 @@ typedef struct {
         mfxU64  reserved2;
     };
 } mfxExtMBDisableSkipMap;
+MFX_PACK_END()
 
 #if (MFX_VERSION >= MFX_VERSION_NEXT)
-
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer    Header;
 
@@ -1613,6 +1705,7 @@ typedef struct {
         mfxU16      reserved[4];
     } DPB[32];
 } mfxExtDPB;
+MFX_PACK_END()
 
 #endif
 
@@ -1642,7 +1735,8 @@ enum {
 
 #endif
 
-#pragma pack(push, 4)
+/* This struct has 4-byte alignment for binary compatibility with previously released versions of API */
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer    Header;
 
@@ -1657,7 +1751,7 @@ typedef struct {
     mfxU16          reserved[118];
 #endif
 } mfxExtHEVCParam;
-#pragma pack(pop)
+MFX_PACK_END()
 
 #if (MFX_VERSION >= 1025)
 /*ErrorTypes in mfxExtDecodeErrorReport*/
@@ -1669,22 +1763,27 @@ enum {
     MFX_ERROR_FRAME_GAP     = (1 << 4),
 };
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer    Header;
 
     mfxU32          ErrorTypes;
     mfxU16          reserved[10];
 } mfxExtDecodeErrorReport;
+MFX_PACK_END()
 
 #endif
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer Header;
 
     mfxU16       FrameType;
     mfxU16       reserved[59];
 } mfxExtDecodedFrameInfo;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer Header;
 
@@ -1695,6 +1794,7 @@ typedef struct {
     mfxU16       TimeCodePictures;
     mfxU16       reserved[7];
 } mfxExtTimeCode;
+MFX_PACK_END()
 
 /*RegionType*/
 enum {
@@ -1707,6 +1807,7 @@ enum {
     MFX_HEVC_REGION_ENCODING_OFF = 1
 };
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer Header;
 
@@ -1715,7 +1816,9 @@ typedef struct {
     mfxU16       RegionEncoding;
     mfxU16       reserved[24];
 } mfxExtHEVCRegion;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer Header;
 
@@ -1726,8 +1829,10 @@ typedef struct {
     mfxI16       Weights[2][32][3][2];      // [list][list entry][Y, Cb, Cr][weight, offset]
     mfxU16       reserved[58];
 } mfxExtPredWeightTable;
+MFX_PACK_END()
 
 #if (MFX_VERSION >= 1027)
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer Header;
 
@@ -1738,10 +1843,11 @@ typedef struct {
 
     mfxU16       reserved[24];
 } mfxExtAVCRoundingOffset;
+MFX_PACK_END()
 #endif
 
 #if (MFX_VERSION >= MFX_VERSION_NEXT)
-
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer Header;
 
@@ -1759,9 +1865,11 @@ typedef struct {
         mfxU16   reserved1[20];
     } Layer[8];
 } mfxExtTemporalLayers;
+MFX_PACK_END()
 
 #endif
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer Header;
 
@@ -1777,7 +1885,9 @@ typedef struct {
         mfxU16  reserved2[8];
     } Rect[256];
 } mfxExtDirtyRect;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer Header;
 
@@ -1795,15 +1905,17 @@ typedef struct {
         mfxU16  reserved2[4];
     } Rect[256];
 } mfxExtMoveRect;
+MFX_PACK_END()
 
 #if (MFX_VERSION >= MFX_VERSION_NEXT)
 
 /* ScalingMatrixType */
 enum {
-    MFX_SCALING_MATRIX_SPS = 0,
-    MFX_SCALING_MATRIX_PPS = 1
+    MFX_SCALING_MATRIX_SPS = 1,
+    MFX_SCALING_MATRIX_PPS = 2
 };
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer Header;
 
@@ -1824,7 +1936,9 @@ typedef struct {
         Inter_Cb, Intra_Cr, Inter_Cr] */
     mfxU8  ScalingList8x8[6][64];
 } mfxExtAVCScalingMatrix;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer Header;
 
@@ -1833,6 +1947,7 @@ typedef struct {
     mfxU8  LoadMatrix[4]; // [LumaIntra, LumaInter, ChromaIntra, ChromaInter]
     mfxU8  Matrix[4][64]; // [LumaIntra, LumaInter, ChromaIntra, ChromaInter]
 } mfxExtMPEG2QuantMatrix;
+MFX_PACK_END()
 
 #endif
 
@@ -1844,13 +1959,16 @@ enum {
     MFX_ANGLE_270   = 270
 };
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer Header;
 
     mfxU16 Angle;
     mfxU16 reserved[11];
 } mfxExtVPPRotation;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_STRUCT_W_L_TYPE()
 typedef struct {
     mfxExtBuffer Header;
 
@@ -1865,6 +1983,7 @@ typedef struct {
 
     mfxU16 reserved[20];
 } mfxExtEncodedSlicesInfo;
+MFX_PACK_END()
 
 /* ScalingMode */
 enum {
@@ -1873,12 +1992,14 @@ enum {
     MFX_SCALING_MODE_QUALITY    = 2
 };
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer Header;
 
     mfxU16 ScalingMode;
     mfxU16 reserved[11];
 } mfxExtVPPScaling;
+MFX_PACK_END()
 
 #if (MFX_VERSION >= MFX_VERSION_NEXT)
 
@@ -1889,12 +2010,14 @@ enum {
     MFX_SCENE_END       = 2
 };
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer Header;
 
     mfxU16 Type;
     mfxU16 reserved[11];
 } mfxExtSceneChange;
+MFX_PACK_END()
 
 #endif
 
@@ -1910,13 +2033,16 @@ enum
     MFX_MIRRORING_VERTICAL   = 2
 };
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer Header;
 
     mfxU16 Type;
     mfxU16 reserved[11];
 } mfxExtVPPMirroring;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer Header;
 
@@ -1926,13 +2052,16 @@ typedef struct {
     mfxU16 StickRight;   /* tri-state option */
     mfxU16 reserved[8];
 } mfxExtMVOverPicBoundaries;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer Header;
 
     mfxU16 Enable;        /* tri-state option */
     mfxU16 reserved[11];
 } mfxExtVPPColorFill;
+MFX_PACK_END()
 
 #if (MFX_VERSION >= 1025)
 
@@ -1946,12 +2075,14 @@ enum {
     MFX_CHROMA_SITING_HORIZONTAL_CENTER   = 0x0020  /* Chroma samples are not co-sited horizontally with the luma samples. */
 };
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer Header;
 
     mfxU16 ChromaSiting;
     mfxU16 reserved[27];
 } mfxExtColorConversion;
+MFX_PACK_END()
 
 #endif
 
@@ -1981,6 +2112,7 @@ enum {
     MFX_VP9_SEGMENT_FEATURE_SKIP        = 0x0008 /* (0,0) MV, no residual */
 };
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxU16  FeatureEnabled;         /* see enum SegmentFeature */
     mfxI16  QIndexDelta;
@@ -1988,7 +2120,9 @@ typedef struct {
     mfxU16  ReferenceFrame;        /* see enum VP9ReferenceFrame */
     mfxU16  reserved[12];
 } mfxVP9SegmentParam;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_STRUCT_W_L_TYPE()
 typedef struct {
     mfxExtBuffer        Header;
     mfxU16              NumSegments;            /* 0..8 */
@@ -2001,19 +2135,25 @@ typedef struct {
     };
     mfxU16  reserved[52];
 } mfxExtVP9Segmentation;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxU16 FrameRateScale;  /* Layer[n].FrameRateScale = Layer[n - 1].FrameRateScale * (uint)m */
     mfxU16 TargetKbps;      /* affected by BRCParamMultiplier, Layer[n].TargetKbps > Layer[n - 1].TargetKbps */
     mfxU16 reserved[14];
 } mfxVP9TemporalLayer;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer        Header;
     mfxVP9TemporalLayer Layer[8];
     mfxU16              reserved[60];
 } mfxExtVP9TemporalLayers;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer Header;
 
@@ -2031,15 +2171,15 @@ typedef struct {
     mfxI16  QIndexDeltaLumaDC;
     mfxI16  QIndexDeltaChromaAC;
     mfxI16  QIndexDeltaChromaDC;
-#if (MFX_VERSION >= MFX_VERSION_NEXT)
+#if (MFX_VERSION >= 1029)
     mfxU16  NumTileRows;
     mfxU16  NumTileColumns;
-    mfxU16  DynamicScaling;
-    mfxU16  reserved[109];
-#else // API 1.26
+    mfxU16  reserved[110];
+#else
     mfxU16  reserved[112];
 #endif
 } mfxExtVP9Param;
+MFX_PACK_END()
 
 #endif // #if (MFX_VERSION >= 1026)
 
@@ -2053,6 +2193,7 @@ enum {
 };
 
 /* Multi-Frame Initialization parameters */
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer Header;
 
@@ -2061,8 +2202,10 @@ typedef struct {
 
     mfxU16      reserved[58];
 } mfxExtMultiFrameParam;
+MFX_PACK_END()
 
 /* Multi-Frame Run-time controls */
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer Header;
 
@@ -2071,7 +2214,9 @@ typedef struct {
 
     mfxU16      reserved[57];
 } mfxExtMultiFrameControl;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxU16 Type;
     mfxU16 reserved1;
@@ -2079,7 +2224,9 @@ typedef struct {
     mfxU32 Size;
     mfxU32 reserved[5];
 } mfxEncodedUnitInfo;
+MFX_PACK_END()
 
+MFX_PACK_BEGIN_STRUCT_W_L_TYPE()
 typedef struct {
     mfxExtBuffer Header;
 
@@ -2092,6 +2239,7 @@ typedef struct {
 
     mfxU16 reserved[22];
 } mfxExtEncodedUnitsInfo;
+MFX_PACK_END()
 
 #endif
 
@@ -2108,6 +2256,7 @@ enum {
 #endif
 
 /* MCTF initialization & runtime */
+MFX_PACK_BEGIN_USUAL_STRUCT()
 typedef struct {
     mfxExtBuffer Header;
     mfxU16       FilterStrength;
@@ -2122,9 +2271,72 @@ typedef struct {
     mfxU16       reserved[27];
 #endif
 } mfxExtVppMctf;
+MFX_PACK_END()
 
 #endif
 
+#if (MFX_VERSION >= 1031)
+/* Multi-adapters Querying structs */
+typedef enum
+{
+    MFX_COMPONENT_ENCODE = 1,
+    MFX_COMPONENT_DECODE = 2,
+    MFX_COMPONENT_VPP    = 3
+} mfxComponentType;
+
+MFX_PACK_BEGIN_STRUCT_W_PTR()
+typedef struct
+{
+    mfxComponentType Type;
+    mfxVideoParam    Requirements;
+
+    mfxU16           reserved[4];
+} mfxComponentInfo;
+MFX_PACK_END()
+
+/* Adapter description */
+MFX_PACK_BEGIN_USUAL_STRUCT()
+typedef struct
+{
+    mfxPlatform Platform;
+    mfxU32      Number;
+
+    mfxU16      reserved[14];
+} mfxAdapterInfo;
+MFX_PACK_END()
+
+MFX_PACK_BEGIN_STRUCT_W_PTR()
+typedef struct
+{
+    mfxAdapterInfo * Adapters;
+    mfxU32           NumAlloc;
+    mfxU32           NumActual;
+
+    mfxU16           reserved[4];
+} mfxAdaptersInfo;
+MFX_PACK_END()
+
+#endif
+
+#if (MFX_VERSION >= 1031)
+/* PartialBitstreamOutput */
+enum {
+    MFX_PARTIAL_BITSTREAM_NONE    = 0,     /* Don't use partial output */
+    MFX_PARTIAL_BITSTREAM_SLICE   = 1,     /* Partial bitstream output will be aligned to slice granularity */
+    MFX_PARTIAL_BITSTREAM_BLOCK   = 2,     /* Partial bitstream output will be aligned to user-defined block size granularity */
+    MFX_PARTIAL_BITSTREAM_ANY     = 3      /* Partial bitstream output will be return any coded data avilable at the end of SyncOperation timeout */
+};
+
+MFX_PACK_BEGIN_USUAL_STRUCT()
+typedef struct {
+    mfxExtBuffer    Header;
+    mfxU32          BlockSize;        /* output block granulatiry for Granularity = MFX_PARTIAL_BITSTREAM_BLOCK */
+    mfxU16          Granularity;      /* granulatiry of the partial bitstream: slice/block/any */
+    mfxU16          reserved[8];
+} mfxExtPartialBitstreamParam;
+MFX_PACK_END()
+#endif
+
 #ifdef __cplusplus
 } // extern "C"
 #endif
