@@ -124,6 +124,8 @@ CBuffering::FreeBuffers()
     m_UsedSurfacesPool.m_pSurfacesTail = NULL;
     m_UsedVppSurfacesPool.m_pSurfacesHead = NULL;
     m_UsedVppSurfacesPool.m_pSurfacesTail = NULL;
+    m_OutputSurfacesPool.m_pSurfacesHead = NULL;
+    m_OutputSurfacesPool.m_pSurfacesTail = NULL;
 
     m_FreeSurfacesPool.m_pSurfaces = NULL;
     m_FreeVppSurfacesPool.m_pSurfaces = NULL;
