@@ -46,7 +46,7 @@ int get_drm_driver_name(int fd, char *name, int name_size)
     return ioctl(fd, DRM_IOWR(0, drm_version), &version);
 }
 
-int open_intel_adapter(int type)
+int open_first_intel_adapter(int type)
 {
     std::string adapterPath = MFX_DRI_PATH;
     char driverName[MFX_DRM_DRIVER_NAME_LEN + 1] = {};
@@ -82,13 +82,37 @@ int open_intel_adapter(int type)
     return -1;
 }
 
-DRMLibVA::DRMLibVA(int type)
+int open_intel_adapter(const std::string& devicePath, int type)
+{
+    if(devicePath.empty())
+        return open_first_intel_adapter(type);
+
+    int fd = open(devicePath.c_str(), O_RDWR);
+
+    if (fd < 0) {
+        msdk_printf(MSDK_STRING("Failed to open specified device\n"));
+        return -1;
+    }
+
+    char driverName[MFX_DRM_DRIVER_NAME_LEN + 1] = {};
+    if (!get_drm_driver_name(fd, driverName, MFX_DRM_DRIVER_NAME_LEN) &&
+        !strcmp(driverName, MFX_DRM_INTEL_DRIVER_NAME)) {
+            return fd;
+    }
+    else {
+        close(fd);
+        msdk_printf(MSDK_STRING("Specified device is not Intel one\n"));
+        return -1;
+    }
+}
+
+DRMLibVA::DRMLibVA(const std::string& devicePath, int type)
     : CLibVA(type)
     , m_fd(-1)
 {
     mfxStatus sts = MFX_ERR_NONE;
 
-    m_fd = open_intel_adapter(type);
+    m_fd = open_intel_adapter(devicePath, type);
     if (m_fd < 0) throw std::range_error("Intel GPU was not found");
 
     m_va_dpy = m_vadrmlib.vaGetDisplayDRM(m_fd);
@@ -182,7 +206,6 @@ drmRenderer::drmRenderer(int fd, mfxI32 monitorType)
 {
     bool res = false;
     uint32_t connectorType = getConnectorType(monitorType);
-    uint32_t i;
 
     if (monitorType == MFX_MONITOR_AUTO) {
       connectorType = DRM_MODE_CONNECTOR_Unknown;
