@@ -48,6 +48,10 @@ or https://software.intel.com/en-us/media-client-solutions-support.
 #include "plugin_loader.h"
 #include "general_allocator.h"
 
+#if defined(_WIN64) || defined(_WIN32)
+#include "mfxadapter.h"
+#endif
+
 #ifndef MFX_VERSION
 #error MFX_VERSION not defined
 #endif
@@ -123,18 +127,18 @@ struct sInputParams
     mfxI32  libvaBackend;
 #endif // defined(MFX_LIBVA_SUPPORT)
 
+#if defined(LINUX32) || defined(LINUX64)
+    std::string strDevicePath;
+#endif
+#if (defined(_WIN64) || defined(_WIN32)) && (MFX_VERSION >= 1031)
+    bool bPrefferdGfx;
+    bool bPrefferiGfx;
+#endif
+
     msdk_char     strSrcFile[MSDK_MAX_FILENAME_LEN];
     msdk_char     strDstFile[MSDK_MAX_FILENAME_LEN];
     sPluginParams pluginParams;
 
-    sInputParams()
-    {
-        MSDK_ZERO_MEMORY(*this);
-    }
-};
-
-template<>struct mfx_ext_buffer_id<mfxExtMVCSeqDesc>{
-    enum {id = MFX_EXTBUFF_MVC_SEQ_DESC};
 };
 
 struct CPipelineStatistics
@@ -181,7 +185,6 @@ public:
     virtual mfxStatus ResetDevice();
 
     void SetMultiView();
-    void SetExtBuffersFlag()       { m_bIsExtBuffers = true; }
     virtual void PrintInfo();
     mfxU64 GetTotalBytesProcessed() { return totalBytesProcessed + m_mfxBS.DataOffset; }
 
@@ -207,21 +210,19 @@ public:
 #endif
 
 protected: // functions
+#if (defined(_WIN64) || defined(_WIN32)) && (MFX_VERSION >= 1031)
+    mfxU32    GetPreferredAdapterNum(const mfxAdaptersInfo & adapters, const sInputParams & params);
+#endif
+    mfxStatus GetImpl(const sInputParams & params, mfxIMPL & impl);
     virtual mfxStatus CreateRenderingWindow(sInputParams *pParams);
     virtual mfxStatus InitMfxParams(sInputParams *pParams);
 
-    // function for allocating a specific external buffer
-    template <typename Buffer>
-    mfxStatus AllocateExtBuffer();
-    virtual void DeleteExtBuffers();
-
     virtual mfxStatus AllocateExtMVCBuffers();
+
     virtual void    DeallocateExtMVCBuffers();
 
-    virtual void AttachExtParam();
-
     virtual mfxStatus InitVppParams();
-    virtual mfxStatus AllocAndInitVppFilters();
+    virtual mfxStatus InitVppFilters();
     virtual bool IsVppRequired(sInputParams *pParams);
 
     virtual mfxStatus CreateAllocator();
@@ -243,29 +244,22 @@ protected: // functions
 
     virtual void DeliverLoop();
 
+    virtual mfxStatus ReallocCurrentSurface(const mfxFrameInfo & info);
+
 protected: // variables
-    CSmplYUVWriter          m_FileWriter;
+    CSmplYUVWriter                         m_FileWriter;
     std::unique_ptr<CSmplBitstreamReader>  m_FileReader;
-    mfxBitstream            m_mfxBS; // contains encoded data
+    mfxBitstreamWrapper                    m_mfxBS; // contains encoded data
     mfxU64 totalBytesProcessed;
 
     MFXVideoSession         m_mfxSession;
     mfxIMPL                 m_impl;
     MFXVideoDECODE*         m_pmfxDEC;
     MFXVideoVPP*            m_pmfxVPP;
-    mfxVideoParam           m_mfxVideoParams;
-    mfxVideoParam           m_mfxVppVideoParams;
+    MfxVideoParamsWrapper   m_mfxVideoParams;
+    MfxVideoParamsWrapper   m_mfxVppVideoParams;
     std::unique_ptr<MFXVideoUSER>  m_pUserModule;
     std::unique_ptr<MFXPlugin> m_pPlugin;
-    std::vector<mfxExtBuffer *> m_ExtBuffers;
-    std::vector<mfxExtBuffer *> m_ExtBuffersMfxBS;
-#if MFX_VERSION >= 1022
-    mfxExtDecVideoProcessing m_DecoderPostProcessing;
-#endif //MFX_VERSION >= 1022
-
-#if (MFX_VERSION >= 1025)
-    mfxExtDecodeErrorReport m_DecodeErrorReport;
-#endif
 
     GeneralAllocator*       m_pGeneralAllocator;
     mfxAllocatorParams*     m_pmfxAllocatorParams;
@@ -287,7 +281,6 @@ protected: // variables
 
     eWorkMode               m_eWorkMode; // work mode for the pipeline
     bool                    m_bIsMVC; // enables MVC mode (need to support several files as an output)
-    bool                    m_bIsExtBuffers; // indicates if external buffers were allocated
     bool                    m_bIsVideoWall; // indicates special mode: decoding will be done in a loop
     bool                    m_bIsCompleteFrame;
     mfxU32                  m_fourcc; // color format of vpp out, i420 by default
@@ -310,13 +303,12 @@ protected: // variables
     msdk_tick               m_startTick;
     msdk_tick               m_delayTicks;
 
-    mfxExtVPPDoNotUse       m_VppDoNotUse;      // for disabling VPP algorithms
-    mfxExtVPPDeinterlacing  m_VppDeinterlacing;
-    std::vector<mfxExtBuffer*> m_VppExtParams;
-
     mfxExtVPPVideoSignalInfo m_VppVideoSignalInfo;
     std::vector<mfxExtBuffer*> m_VppSurfaceExtParams;
 
+#if defined(LINUX32) || defined(LINUX64)
+    std::string              m_strDevicePath; //path to device for processing
+#endif
     CHWDevice               *m_hwdev;
 #if D3D_SURFACES_SUPPORT
     CDecodeD3DRender         m_d3dRender;
