@@ -132,13 +132,11 @@ public:
         , m_session(session)
         , m_LastSyncp(0)
     {
-        MSDK_ZERO_MEMORY(m_Bitstream);
         m_Bitstream.TimeStamp=(mfxU64)-1;
     }
 
     virtual ~Decoder()
     {
-        WipeMfxBitstream(&m_Bitstream);
     }
 
     virtual mfxStatus QueryIOSurf(mfxFrameAllocRequest* request);
@@ -155,7 +153,7 @@ protected:
 
 protected:
     CSmplBitstreamReader             m_FileReader;
-    mfxBitstream                     m_Bitstream;
+    mfxBitstreamWrapper              m_Bitstream;
 
     MFXVideoSession*                 m_session;
     std::unique_ptr<MFXVideoDECODE>  m_DEC;
